@@ -31,6 +31,9 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#include <config.h>
+
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <sys/socket.h>
@@ -83,7 +86,7 @@ static char *remove_whitespace(char *string)
 
 static int parse_section(FILE *fp,
 			 struct objdb_iface_ver0 *objdb,
-			 unsigned int parent_handle,
+			 hdb_handle_t parent_handle,
 			 char **error_string)
 {
 	char line[512];
@@ -91,7 +94,8 @@ static int parse_section(FILE *fp,
 	char *loc;
 
 	while (fgets (line, 255, fp)) {
-		line[strlen(line) - 1] = '\0';
+		if (strlen(line) > 0)
+			line[strlen(line) - 1] = '\0';
 		/*
 		 * Clear out white space and tabs
 		 */
@@ -111,7 +115,7 @@ static int parse_section(FILE *fp,
 
 		/* New section ? */
 		if ((loc = strstr_rs (line, "{"))) {
-			unsigned int new_parent;
+			hdb_handle_t new_parent;
 			char *section = remove_whitespace(line);
 
 			loc--;
@@ -162,7 +166,7 @@ static int read_config_file_into_objdb(
 
 	filename = getenv ("COROSYNC_MAIN_CONFIG_FILE");
 	if (!filename)
-		filename = "/etc/corosync.conf";
+		filename = SYSCONFDIR "/corosync.conf";
 
 	fp = fopen (filename, "r");
 	if (fp == 0) {
