@@ -32,6 +32,9 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#include <config.h>
+
 #include <errno.h>
 #include <pthread.h>
 #include <sys/poll.h>
@@ -39,12 +42,12 @@
 #include <string.h>
 #include <stdio.h>
 
+#include <corosync/hdb.h>
 #include <corosync/totem/coropoll.h>
 #include <corosync/list.h>
-#include <corosync/hdb.h>
 #include "tlist.h"
 
-typedef int (*dispatch_fn_t) (poll_handle poll_handle, int fd, int revents, void *data);
+typedef int (*dispatch_fn_t) (hdb_handle_t hdb_handle_t, int fd, int revents, void *data);
 
 struct poll_entry {
 	struct pollfd ufd;
@@ -57,8 +60,7 @@ struct poll_instance {
 	struct pollfd *ufds;
 	int poll_entry_count;
 	struct timerlist timerlist;
-	void (*serialize_lock_fn) (void);
-	void (*serialize_unlock_fn) (void);
+	int stop_requested;
 };
 
 /*
@@ -70,11 +72,9 @@ static struct hdb_handle_database poll_instance_database = {
 	.iterator	= 0
 };
 
-poll_handle poll_create (
-	void (*serialize_lock_fn) (void),
-	void (*serialize_unlock_fn) (void))
+hdb_handle_t poll_create (void)
 {
-	poll_handle handle;
+	hdb_handle_t handle;
 	struct poll_instance *poll_instance;
 	unsigned int res;
 
@@ -92,8 +92,7 @@ poll_handle poll_create (
 	poll_instance->poll_entries = 0;
 	poll_instance->ufds = 0;
 	poll_instance->poll_entry_count = 0;
-	poll_instance->serialize_lock_fn = serialize_lock_fn;
-	poll_instance->serialize_unlock_fn = serialize_unlock_fn;
+	poll_instance->stop_requested = 0;
 	timerlist_init (&poll_instance->timerlist);
 
 	return (handle);
@@ -105,7 +104,7 @@ error_exit:
 	return (-1);
 }
 
-int poll_destroy (poll_handle handle)
+int poll_destroy (hdb_handle_t handle)
 {
 	struct poll_instance *poll_instance;
 	int res = 0;
@@ -133,12 +132,12 @@ error_exit:
 }
 
 int poll_dispatch_add (
-	poll_handle handle,
+	hdb_handle_t handle,
 	int fd,
 	int events,
 	void *data,
 	int (*dispatch_fn) (
-		poll_handle poll_handle,
+		hdb_handle_t hdb_handle_t,
 		int fd,
 		int revents,
 		void *data))
@@ -207,11 +206,11 @@ error_exit:
 }
 
 int poll_dispatch_modify (
-	poll_handle handle,
+	hdb_handle_t handle,
 	int fd,
 	int events,
 	int (*dispatch_fn) (
-		poll_handle poll_handle,
+		hdb_handle_t hdb_handle_t,
 		int fd,
 		int revents,
 		void *data))
@@ -249,7 +248,7 @@ error_exit:
 }
 
 int poll_dispatch_delete (
-	poll_handle handle,
+	hdb_handle_t handle,
 	int fd)
 {
 	struct poll_instance *poll_instance;
@@ -283,7 +282,7 @@ error_exit:
 }
 
 int poll_timer_add (
-	poll_handle handle,
+	hdb_handle_t handle,
 	int msec_duration, void *data,
 	void (*timer_fn) (void *data),
 	poll_timer_handle *timer_handle_out)
@@ -291,18 +290,18 @@ int poll_timer_add (
 	struct poll_instance *poll_instance;
 	int res = 0;
 
+	if (timer_handle_out == NULL) {
+		res = -ENOENT;
+		goto error_exit;
+	}
+
 	res = hdb_handle_get (&poll_instance_database, handle,
 		(void *)&poll_instance);
 	if (res != 0) {
 		res = -ENOENT;
-		
 		goto error_exit;
 	}
 
-	if (timer_handle_out == 0) {
-		res = -ENOENT;
-	}
-
 	timerlist_add_duration (&poll_instance->timerlist,
 		timer_fn, data, ((unsigned long long)msec_duration) * 1000000ULL, timer_handle_out);
 
@@ -312,7 +311,7 @@ error_exit:
 }
 
 int poll_timer_delete (
-	poll_handle handle,
+	hdb_handle_t handle,
 	poll_timer_handle timer_handle)
 {
 	struct poll_instance *poll_instance;
@@ -336,8 +335,29 @@ error_exit:
 	return (res);
 }
 
+int poll_stop (
+	hdb_handle_t handle)
+{
+	struct poll_instance *poll_instance;
+	unsigned int res;
+
+	res = hdb_handle_get (&poll_instance_database, handle,
+		(void *)&poll_instance);
+	if (res != 0) {
+		res = -ENOENT;
+		goto error_exit;
+	}
+	
+	poll_instance->stop_requested = 1;
+
+	hdb_handle_put (&poll_instance_database, handle);
+error_exit:
+	return (res);
+}
+
+
 int poll_run (
-	poll_handle handle)
+	hdb_handle_t handle)
 {
 	struct poll_instance *poll_instance;
 	int i;
@@ -366,6 +386,10 @@ int poll_run (
 retry_poll:
 		res = poll (poll_instance->ufds,
 			poll_instance->poll_entry_count, expire_timeout_msec);
+		if (poll_instance->stop_requested) {
+			printf ("poll should stop\n");
+			return (0);
+		}
 		if (errno == EINTR && res == -1) {
 			goto retry_poll;
 		} else
@@ -378,13 +402,11 @@ retry_poll:
 			if (poll_instance->ufds[i].fd != -1 &&
 				poll_instance->ufds[i].revents) {
 
-				poll_instance->serialize_lock_fn();
 				res = poll_instance->poll_entries[i].dispatch_fn (handle,
 					poll_instance->ufds[i].fd, 
 					poll_instance->ufds[i].revents,
 					poll_instance->poll_entries[i].data);
 
-				poll_instance->serialize_unlock_fn();
 				/*
 				 * Remove dispatch functions that return -1
 				 */
@@ -393,9 +415,7 @@ retry_poll:
 				}
 			}
 		}
-		poll_instance->serialize_lock_fn();
 		timerlist_expire (&poll_instance->timerlist);
-		poll_instance->serialize_unlock_fn();
 	} /* for (;;) */
 
 	hdb_handle_put (&poll_instance_database, handle);
@@ -403,12 +423,9 @@ error_exit:
 	return (-1);
 }
 
-int poll_stop (
-	poll_handle handle);
-
 #ifdef COMPILE_OUT
 void poll_print_state (
-	poll_handle handle,
+	hdb_handle_t handle,
 	int fd)
 {
 	struct poll_instance *poll_instance;
