@@ -6,8 +6,11 @@
  * The library is free for all purposes without any express
  * guarantee it works.
  *
- * Tom St Denis, tomstdenis@iahu.ca, http://libtomcrypt.org
+ * Tom St Denis, tomstdenis@iahu.ca, http://libtomcrypt.com
  */
+
+#include <config.h>
+
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
@@ -20,13 +23,14 @@
 #endif
 #include <fcntl.h>
 #include <unistd.h>
+#include <stdint.h>
 
 #include "crypto.h"
 
 #define CONST64(n) n ## ULL
 
-typedef unsigned long ulong32;
-typedef unsigned long long ulong64;
+typedef uint32_t ulong32;
+typedef uint64_t ulong64;
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #define ENDIAN_LITTLE
@@ -36,15 +40,15 @@ typedef unsigned long long ulong64;
 #define ENDIAN_LITTLE
 #elif _BYTE_ORDER == _BIG_ENDIAN
 #define ENDIAN_BIG
-#elif
-#warning "cannot detect byte order"
+#else
+#error "cannot detect byte order"
 #endif
 
 #if defined(COROSYNC_LINUX)
-#if __WORDIZE == 64
+#if __WORDSIZE == 64
 #define ENDIAN_64BITWORD
 #endif
-#if __WORDIZE == 32
+#if __WORDSIZE == 32
 #define ENDIAN_32BITWORD
 #endif
 #else
@@ -681,7 +685,7 @@ const struct _prng_descriptor *prng_descriptor[] = {
 
 #define B(x,i) ((unsigned char)(((x) >> (8*i)) & 0xFF))
 
-static ulong32 BYTE2WORD(unsigned char *b)
+static ulong32 BYTE2WORD(const unsigned char *b)
 {
    ulong32 t;
    LOAD32L(t, b);
@@ -844,7 +848,7 @@ int sober128_add_entropy(const unsigned char *buf, unsigned long len, prng_state
        assert ((len & 3) == 0);
     
        for (i = 0; i < len; i += 4) {
-           k = BYTE2WORD((unsigned char *)&buf[i]);
+           k = BYTE2WORD(&buf[i]);
           ADDKEY(k);
           cycle(c->R);
           XORNL(nltap(c));
@@ -869,7 +873,7 @@ int sober128_add_entropy(const unsigned char *buf, unsigned long len, prng_state
        assert ((len & 3) == 0);
     
        for (i = 0; i < len; i += 4) {
-           k = BYTE2WORD((unsigned char *)&buf[i]);
+           k = BYTE2WORD(&buf[i]);
           ADDKEY(k);
           cycle(c->R);
           XORNL(nltap(c));
@@ -986,7 +990,7 @@ const struct _hash_descriptor sha1_desc =
 #define F2(x,y,z)  ((x & y) | (z & (x | y)))
 #define F3(x,y,z)  (x ^ y ^ z)
 
-static void sha1_compress(hash_state *md, unsigned char *buf)
+static void sha1_compress(hash_state *md, const unsigned char *buf)
 {
     ulong32 a,b,c,d,e,W[80],i;
 
@@ -1250,11 +1254,11 @@ static unsigned long rng_nix(unsigned char *buf, unsigned long len,
                              void (*callback)(void))
 {
 	int fd;
-	int rb;
+	unsigned long rb;
 
 	fd = open ("/dev/urandom", O_RDONLY);
 	
-	rb = read (fd, buf, len);
+	rb = (unsigned long)read (fd, buf, len);
 
 	close (fd);
 
