@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -32,10 +32,11 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#include <config.h>
+
 #include <pthread.h>
 #include <assert.h>
-#include <pwd.h>
-#include <grp.h>
 #include <sys/types.h>
 #include <sys/poll.h>
 #include <sys/uio.h>
@@ -56,7 +57,7 @@
 #include <time.h>
 
 #include <corosync/swab.h>
-#include <corosync/saAis.h>
+#include <corosync/corotypes.h>
 #include <corosync/list.h>
 #include <corosync/queue.h>
 #include <corosync/lcr/lcr_ifact.h>
@@ -66,6 +67,7 @@
 #include <corosync/engine/config.h>
 #include <corosync/engine/logsys.h>
 
+#include "quorum.h"
 #include "totemsrp.h"
 #include "mempool.h"
 #include "mainconfig.h"
@@ -73,31 +75,31 @@
 #include "main.h"
 #include "sync.h"
 #include "tlist.h"
-#include "flow.h"
-#include "ipc.h"
+#include "coroipcs.h"
 #include "timer.h"
 #include "util.h"
-#include "flow.h"
 #include "apidef.h"
 #include "service.h"
 #include "version.h"
 
 LOGSYS_DECLARE_SYSTEM ("corosync",
-	LOG_MODE_OUTPUT_STDERR | LOG_MODE_OUTPUT_SYSLOG_THREADED | LOG_MODE_BUFFER_BEFORE_CONFIG,
+	LOG_MODE_OUTPUT_STDERR | LOG_MODE_THREADED | LOG_MODE_FORK,
 	NULL,
-	LOG_DAEMON);
+	LOG_DAEMON,
+	NULL,
+	1000000);
 
 LOGSYS_DECLARE_SUBSYS ("MAIN", LOG_INFO);
 
 #define SERVER_BACKLOG 5
 
-static int ais_uid = 0;
-
-static int gid_valid = 0;
-
 static unsigned int service_count = 32;
 
+#if defined(HAVE_PTHREAD_SPIN_LOCK)
+static pthread_spinlock_t serialize_spin;
+#else
 static pthread_mutex_t serialize_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif
 
 static struct totem_logging_configuration totem_logging_configuration;
 
@@ -111,8 +113,12 @@ static struct objdb_iface_ver0 *objdb = NULL;
 
 static struct corosync_api_v1 *api = NULL;
 
+static struct main_config main_config;
+
 unsigned long long *(*main_clm_get_by_nodeid) (unsigned int node_id);
 
+hdb_handle_t corosync_poll_handle;
+
 static void sigusr2_handler (int num)
 {
 	int i;
@@ -124,7 +130,7 @@ static void sigusr2_handler (int num)
 	}
 }
 
-static void *aisexec_exit (void *arg)
+static void *corosync_exit (void *arg)
 {
 	if (api) {
 		corosync_service_unlink_all (api);
@@ -144,19 +150,19 @@ static void *aisexec_exit (void *arg)
 	}
 #endif
 
+	poll_stop (0);
 	totempg_finalize ();
-	logsys_flush ();
-
+	coroipcs_ipc_exit ();
 	corosync_exit_error (AIS_DONE_EXIT);
 
 	/* never reached */
 	return NULL;
 }
 
-pthread_t aisexec_exit_thread;
+pthread_t corosync_exit_thread;
 static void init_shutdown(void *data) 
 {
-	pthread_create (&aisexec_exit_thread, NULL, aisexec_exit, NULL);
+	pthread_create (&corosync_exit_thread, NULL, corosync_exit, NULL);
 }
 
 
@@ -164,27 +170,29 @@ static poll_timer_handle shutdown_handle;
 static void sigquit_handler (int num)
 {
 	/* avoid creating threads from within the interrupt context */
-	poll_timer_add (aisexec_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
+	poll_timer_add (corosync_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
 }
 
 
 static void sigsegv_handler (int num)
 {
-	signal (SIGSEGV, SIG_DFL);
-	logsys_flush ();
+	(void)signal (SIGSEGV, SIG_DFL);
+	logsys_atsegv();
+	logsys_log_rec_store (LOCALSTATEDIR "/lib/corosync/fdata");
 	raise (SIGSEGV);
 }
 
 static void sigabrt_handler (int num)
 {
-	signal (SIGABRT, SIG_DFL);
-	logsys_flush ();
+	(void)signal (SIGABRT, SIG_DFL);
+	logsys_atsegv();
+	logsys_log_rec_store (LOCALSTATEDIR "/lib/corosync/fdata");
 	raise (SIGABRT);
 }
 
 #define LOCALHOST_IP inet_addr("127.0.0.1")
 
-totempg_groups_handle corosync_group_handle;
+hdb_handle_t corosync_group_handle;
 
 struct totempg_group corosync_group = {
 	.group		= "a",
@@ -193,7 +201,7 @@ struct totempg_group corosync_group = {
 
 void sigintr_handler (int signum)
 {
-	poll_timer_add (aisexec_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
+	poll_timer_add (corosync_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
 }
 
 
@@ -201,15 +209,28 @@ static int pool_sizes[] = { 0, 0, 0, 0, 0, 4096, 0, 1, 0, /* 256 */
 					1024, 0, 1, 4096, 0, 0, 0, 0, /* 65536 */
 					1, 1, 1, 1, 1, 1, 1, 1, 1 };
 
-void serialize_mutex_lock (void)
+#if defined(HAVE_PTHREAD_SPIN_LOCK)
+void serialize_lock (void)
+{
+	pthread_spin_lock (&serialize_spin);
+}
+
+void serialize_unlock (void)
+{
+	pthread_spin_unlock (&serialize_spin);
+}
+#else
+void serialize_lock (void)
 {
 	pthread_mutex_lock (&serialize_mutex);
 }
 
-void serialize_mutex_unlock (void)
+void serialize_unlock (void)
 {
 	pthread_mutex_unlock (&serialize_mutex);
 }
+#endif
+
 
 
 static void corosync_sync_completed (void)
@@ -245,7 +266,7 @@ static int corosync_sync_callbacks_retrieve (int sync_id,
 	return (0);
 }
 
-static struct memb_ring_id aisexec_ring_id;
+static struct memb_ring_id corosync_ring_id;
 
 static void confchg_fn (
 	enum totem_configuration_type configuration_type,
@@ -256,7 +277,8 @@ static void confchg_fn (
 {
 	int i;
 
-	memcpy (&aisexec_ring_id, ring_id, sizeof (struct memb_ring_id));
+	serialize_lock ();
+	memcpy (&corosync_ring_id, ring_id, sizeof (struct memb_ring_id));
 
 	/*
 	 * Call configuration change for all services
@@ -269,41 +291,17 @@ static void confchg_fn (
 				joined_list, joined_list_entries, ring_id);
 		}
 	}
+	serialize_unlock ();
 }
 
-static void aisexec_uid_determine (struct main_config *main_config)
+static void priv_drop (void)
 {
-	struct passwd *passwd;
-
-	passwd = getpwnam(main_config->user);
-	if (passwd == 0) {
-		log_printf (LOG_LEVEL_ERROR, "ERROR: The '%s' user is not found in /etc/passwd, please read the documentation.\n", main_config->user);
-		corosync_exit_error (AIS_DONE_UID_DETERMINE);
-	}
-	ais_uid = passwd->pw_uid;
-	endpwent ();
+return; /* TODO: we are still not dropping privs */
+	setuid (main_config.uid);
+	setegid (main_config.gid);
 }
 
-static void aisexec_gid_determine (struct main_config *main_config)
-{
-	struct group *group;
-	group = getgrnam (main_config->group);
-	if (group == 0) {
-		log_printf (LOG_LEVEL_ERROR, "ERROR: The '%s' group is not found in /etc/group, please read the documentation.\n", group->gr_name);
-		corosync_exit_error (AIS_DONE_GID_DETERMINE);
-	}
-	gid_valid = group->gr_gid;
-	endgrent ();
-}
-
-static void aisexec_priv_drop (void)
-{
-return;
-	setuid (ais_uid);
-	setegid (ais_uid);
-}
-
-static void aisexec_mempool_init (void)
+static void corosync_mempool_init (void)
 {
 	int res;
 
@@ -314,7 +312,7 @@ static void aisexec_mempool_init (void)
 	}
 }
 
-static void aisexec_tty_detach (void)
+static void corosync_tty_detach (void)
 {
 	int fd;
 
@@ -330,10 +328,11 @@ static void aisexec_tty_detach (void)
 			/*
 			 * child which is disconnected, run this process
 			 */
-/* 			setset(); */
+/* 			setset();
 			close (0);
 			close (1);
 			close (2);
+*/
 			break;
 		default:
 			exit (0);
@@ -341,7 +340,7 @@ static void aisexec_tty_detach (void)
 	}
 
 	/* Create new session */
-	setsid();
+	(void)setsid();
 
 	/* 
 	 * Map stdin/out/err to /dev/null.
@@ -359,7 +358,7 @@ static void aisexec_tty_detach (void)
 	}
 }
 
-static void aisexec_setscheduler (void)
+static void corosync_setscheduler (void)
 {
 #if ! defined(TS_CLASS) && (defined(COROSYNC_BSD) || defined(COROSYNC_LINUX) || defined(COROSYNC_SOLARIS))
 	struct sched_param sched_param;
@@ -380,7 +379,7 @@ static void aisexec_setscheduler (void)
 #endif
 }
 
-static void aisexec_mlockall (void)
+static void corosync_mlockall (void)
 {
 #if !defined(COROSYNC_BSD)
 	int res;
@@ -408,7 +407,6 @@ static void aisexec_mlockall (void)
 #endif
 }
 
-
 static void deliver_fn (
 	unsigned int nodeid,
 	struct iovec *iovec,
@@ -448,6 +446,11 @@ static void deliver_fn (
 	 */
 	service = header->id >> 16;
 	fn_id = header->id & 0xffff;
+	if (!ais_service[service])
+		return;
+
+	serialize_lock();
+
 	if (endian_conversion_required) {
 		assert(ais_service[service]->exec_engine[fn_id].exec_endian_convert_fn != NULL);
 		ais_service[service]->exec_engine[fn_id].exec_endian_convert_fn
@@ -456,6 +459,8 @@ static void deliver_fn (
 
 	ais_service[service]->exec_engine[fn_id].exec_handler_fn
 		(header, nodeid);
+
+	serialize_unlock();
 }
 
 void main_get_config_modules(struct config_iface_ver0 ***modules, int *num)
@@ -472,29 +477,216 @@ int main_mcast (
 	return (totempg_groups_mcast_joined (corosync_group_handle, iovec, iov_len, guarantee));
 }
 
-extern int main_send_ok (
-        struct iovec *iovec,
-        int iov_len)
+int message_source_is_local (mar_message_source_t *source)
 {
-	return (totempg_groups_send_ok_joined (corosync_group_handle, iovec, iov_len));
+	int ret = 0;
+
+	assert (source != NULL);
+	if (source->nodeid == totempg_my_nodeid_get ()) {
+		ret = 1;
+	}
+	return ret;
 }
 
+void message_source_set (
+	mar_message_source_t *source,
+	void *conn)
+{
+	assert ((source != NULL) && (conn != NULL));
+	memset (source, 0, sizeof (mar_message_source_t));
+	source->nodeid = totempg_my_nodeid_get ();
+	source->conn = conn;
+}
+
+/*
+ * Provides the glue from corosync to the IPC Service
+ */
+static int corosync_private_data_size_get (unsigned int service)
+{
+	return (ais_service[service]->private_data_size);
+}
+
+static coroipcs_init_fn_lvalue corosync_init_fn_get (unsigned int service)
+{
+	return (ais_service[service]->lib_init_fn);
+}
+
+static coroipcs_exit_fn_lvalue corosync_exit_fn_get (unsigned int service)
+{
+	return (ais_service[service]->lib_exit_fn);
+}
+
+static coroipcs_handler_fn_lvalue corosync_handler_fn_get (unsigned int service, unsigned int id)
+{
+	return (ais_service[service]->lib_engine[id].lib_handler_fn);
+}
+
+
+static int corosync_security_valid (int euid, int egid)
+{
+	if (euid == 0 || egid == 0) {
+		return (1);
+	}
+	if (euid == main_config.uid || egid == main_config.gid) {
+		return (1);
+	}
+	return (0);
+}
+
+static int corosync_service_available (unsigned int service)
+{
+	return (ais_service[service]);
+}
+
+static int corosync_response_size_get (unsigned int service, unsigned int id)
+{
+	return (ais_service[service]->lib_engine[id].response_size);
+
+}
+
+static int corosync_response_id_get (unsigned int service, unsigned int id)
+{
+	return (ais_service[service]->lib_engine[id].response_id);
+}
+
+
+struct sending_allowed_private_data_struct {
+	int reserved_msgs;
+};
+
+static int corosync_sending_allowed (
+	unsigned int service,
+	unsigned int id, 
+	void *msg,
+	void *sending_allowed_private_data)
+{
+	struct sending_allowed_private_data_struct *pd =
+		(struct sending_allowed_private_data_struct *)sending_allowed_private_data;
+	struct iovec reserve_iovec;
+	mar_req_header_t *header = (mar_req_header_t *)msg;
+	int sending_allowed;
+
+	reserve_iovec.iov_base = (char *)header;
+	reserve_iovec.iov_len = header->size;
+
+	pd->reserved_msgs = totempg_groups_joined_reserve (
+		corosync_group_handle,
+		&reserve_iovec, 1);
+
+	sending_allowed =
+		(corosync_quorum_is_quorate() == 1 ||
+		ais_service[service]->allow_inquorate == CS_LIB_ALLOW_INQUORATE) &&
+		((ais_service[service]->lib_engine[id].flow_control == CS_LIB_FLOW_CONTROL_NOT_REQUIRED) ||
+		((ais_service[service]->lib_engine[id].flow_control == CS_LIB_FLOW_CONTROL_REQUIRED) &&
+		(pd->reserved_msgs) &&
+		(sync_in_process() == 0)));
+
+	return (sending_allowed);
+}
+
+static void corosync_sending_allowed_release (void *sending_allowed_private_data)
+{
+	struct sending_allowed_private_data_struct *pd =
+		(struct sending_allowed_private_data_struct *)sending_allowed_private_data;
+
+	totempg_groups_joined_release (pd->reserved_msgs);
+}
+
+static int ipc_subsys_id = -1;
+
+static void ipc_log_printf (char *format, ...) {
+        va_list ap;
+
+        va_start (ap, format);
+	
+       _logsys_log_printf (ipc_subsys_id, __FUNCTION__,	
+                __FILE__, __LINE__, LOG_LEVEL_ERROR, format, ap);
+
+        va_end (ap);
+}
+
+static int corosync_poll_handler_accept (
+	hdb_handle_t handle,	
+	int fd,
+	int revent,
+	void *context)
+{
+	return (coroipcs_handler_accept (fd, revent, context));
+}
+
+static int corosync_poll_handler_dispatch (
+	hdb_handle_t handle,	
+	int fd,
+	int revent,
+	void *context)
+{
+	return (coroipcs_handler_dispatch (fd, revent, context));
+}
+
+
+static void corosync_poll_accept_add (
+	int fd)
+{
+	poll_dispatch_add (corosync_poll_handle, fd, POLLIN|POLLNVAL, 0,
+		corosync_poll_handler_accept);
+}
+
+static void corosync_poll_dispatch_add (
+	int fd,
+	void *context)
+{
+	poll_dispatch_add (corosync_poll_handle, fd, POLLIN|POLLNVAL, context,
+		corosync_poll_handler_dispatch);
+}
+
+static void corosync_poll_dispatch_modify (
+	int fd,
+	int events)
+{
+	poll_dispatch_modify (corosync_poll_handle, fd, events,
+		corosync_poll_handler_dispatch);
+}
+
+struct coroipcs_init_state ipc_init_state = {
+	.socket_name			= IPC_SOCKET_NAME,
+	.malloc				= malloc,
+	.free				= free,
+	.log_printf			= ipc_log_printf,
+	.security_valid			= corosync_security_valid,
+	.service_available		= corosync_service_available,
+	.private_data_size_get		= corosync_private_data_size_get,
+	.serialize_lock			= serialize_lock,
+	.serialize_unlock		= serialize_unlock,
+	.sending_allowed		= corosync_sending_allowed,
+	.sending_allowed_release	= corosync_sending_allowed_release,
+	.response_size_get		= corosync_response_size_get,
+	.response_id_get		= corosync_response_id_get,
+	.poll_accept_add		= corosync_poll_accept_add,
+	.poll_dispatch_add		= corosync_poll_dispatch_add,
+	.poll_dispatch_modify		= corosync_poll_dispatch_modify,
+	.init_fn_get			= corosync_init_fn_get,
+	.exit_fn_get			= corosync_exit_fn_get,
+	.handler_fn_get			= corosync_handler_fn_get
+};
+
 int main (int argc, char **argv)
 {
-	char *error_string;
-	struct main_config main_config;
+	const char *error_string;
 	struct totem_config totem_config;
-	unsigned int objdb_handle;
-	unsigned int config_handle;
+	hdb_handle_t objdb_handle;
+	hdb_handle_t config_handle;
 	unsigned int config_version = 0;
 	void *objdb_p;
 	struct config_iface_ver0 *config;
 	void *config_p;
-	char *config_iface;
+	const char *config_iface;
 	char *iface;
 	int res, ch;
 	int background, setprio;
- 	int totem_log_service;
+
+#if defined(HAVE_PTHREAD_SPIN_LOCK)
+	pthread_spin_init (&serialize_spin, 0);
+#endif
 
  	/* default configuration
 	 */
@@ -506,7 +698,7 @@ int main (int argc, char **argv)
 		switch (ch) {
 			case 'f':
 				background = 0;
-				logsys_config_mode_set (LOG_MODE_OUTPUT_STDERR|LOG_MODE_FLUSH_AFTER_CONFIG);
+				logsys_config_mode_set (LOG_MODE_OUTPUT_STDERR|LOG_MODE_THREADED|LOG_MODE_FORK);
 				break;
 			case 'p':
 				setprio = 0;
@@ -521,27 +713,25 @@ int main (int argc, char **argv)
 	}	
 
 	if (background)
-		aisexec_tty_detach ();
+		corosync_tty_detach ();
 
 	log_printf (LOG_LEVEL_NOTICE, "Corosync Executive Service RELEASE '%s'\n", RELEASE_VERSION);
 	log_printf (LOG_LEVEL_NOTICE, "Copyright (C) 2002-2006 MontaVista Software, Inc and contributors.\n");
 	log_printf (LOG_LEVEL_NOTICE, "Copyright (C) 2006-2008 Red Hat, Inc.\n");
 
-	signal (SIGINT, sigintr_handler);
-	signal (SIGUSR2, sigusr2_handler);
-	signal (SIGSEGV, sigsegv_handler);
-	signal (SIGABRT, sigabrt_handler);
-	signal (SIGQUIT, sigquit_handler);
+	(void)signal (SIGINT, sigintr_handler);
+	(void)signal (SIGUSR2, sigusr2_handler);
+	(void)signal (SIGSEGV, sigsegv_handler);
+	(void)signal (SIGABRT, sigabrt_handler);
+	(void)signal (SIGQUIT, sigquit_handler);
 	
 	corosync_timer_init (
-		serialize_mutex_lock,
-		serialize_mutex_unlock);
+		serialize_lock,
+		serialize_unlock);
 
 	log_printf (LOG_LEVEL_NOTICE, "Corosync Executive Service: started and ready to provide service.\n");
 
-	aisexec_poll_handle = poll_create (
-		serialize_mutex_lock,
-		serialize_mutex_unlock);
+	corosync_poll_handle = poll_create ();
 
 	/*
 	 * Load the object database interface
@@ -600,10 +790,10 @@ int main (int argc, char **argv)
 
 		res = config->config_readconfig(objdb, &error_string);
 		if (res == -1) {
-			log_printf (LOG_LEVEL_ERROR, error_string);
+			log_printf (LOG_LEVEL_ERROR, "%s", error_string);
 			corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 		}
-		log_printf (LOG_LEVEL_NOTICE, error_string);
+		log_printf (LOG_LEVEL_NOTICE, "%s", error_string);
 		config_modules[num_config_modules++] = config;
 
 		iface = strtok(NULL, ":");
@@ -613,59 +803,52 @@ int main (int argc, char **argv)
 
 	res = corosync_main_config_read (objdb, &error_string, &main_config);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, error_string);
+		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
 	res = totem_config_read (objdb, &totem_config, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, error_string);
+		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
 	res = totem_config_keyread (objdb, &totem_config, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, error_string);
+		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
 	res = totem_config_validate (&totem_config, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, error_string);
+		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
-	logsys_config_facility_set ("corosync", main_config.syslog_facility);
-	logsys_config_mode_set (main_config.logmode);
-	logsys_config_file_set (&error_string, main_config.logfile);
-
-	aisexec_uid_determine (&main_config);
-
-	aisexec_gid_determine (&main_config);
-
 	/*
 	 * Set round robin realtime scheduling with priority 99
 	 * Lock all memory to avoid page faults which may interrupt
 	 * application healthchecking
 	 */
 	if (setprio)
-		aisexec_setscheduler ();
+		corosync_setscheduler ();
 
-	aisexec_mlockall ();
+	corosync_mlockall ();
 
 	totem_config.totem_logging_configuration = totem_logging_configuration;
-	totem_log_service = _logsys_subsys_create ("TOTEM", LOG_INFO);
-  	totem_config.totem_logging_configuration.log_level_security = logsys_mkpri (LOG_LEVEL_SECURITY, totem_log_service);
-	totem_config.totem_logging_configuration.log_level_error = logsys_mkpri (LOG_LEVEL_ERROR, totem_log_service);
-	totem_config.totem_logging_configuration.log_level_warning = logsys_mkpri (LOG_LEVEL_WARNING, totem_log_service);
-	totem_config.totem_logging_configuration.log_level_notice = logsys_mkpri (LOG_LEVEL_NOTICE, totem_log_service);
-	totem_config.totem_logging_configuration.log_level_debug = logsys_mkpri (LOG_LEVEL_DEBUG, totem_log_service);
-	totem_config.totem_logging_configuration.log_printf = logsys_log_printf;
+	totem_config.totem_logging_configuration.log_subsys_id =
+		_logsys_subsys_create ("TOTEM", LOG_INFO);
+  	totem_config.totem_logging_configuration.log_level_security = LOG_LEVEL_SECURITY;
+	totem_config.totem_logging_configuration.log_level_error = LOG_LEVEL_ERROR;
+	totem_config.totem_logging_configuration.log_level_warning = LOG_LEVEL_WARNING;
+	totem_config.totem_logging_configuration.log_level_notice = LOG_LEVEL_NOTICE;
+	totem_config.totem_logging_configuration.log_level_debug = LOG_LEVEL_DEBUG;
+	totem_config.totem_logging_configuration.log_printf = _logsys_log_printf;
 
 	/*
 	 * Sleep for a while to let other nodes in the cluster
 	 * understand that this node has been away (if it was
-	 * an aisexec restart).
+	 * an corosync restart).
 	 */
 
 // TODO what is this hack for?	usleep(totem_config.token_timeout * 2000);
@@ -681,7 +864,7 @@ int main (int argc, char **argv)
 	 *  and configuration change functions
 	 */
 	totempg_initialize (
-		aisexec_poll_handle,
+		corosync_poll_handle,
 		&totem_config);
 
 	totempg_groups_initialize (
@@ -704,11 +887,7 @@ int main (int argc, char **argv)
 	}
 
 
-	sync_register (corosync_sync_callbacks_retrieve, corosync_sync_completed,
-		totem_config.vsf_type);
-
-
-	res = corosync_flow_control_initialize ();
+	sync_register (corosync_sync_callbacks_retrieve, corosync_sync_completed);
 
 	/*
 	 * Drop root privleges to user 'ais'
@@ -718,19 +897,18 @@ int main (int argc, char **argv)
 	 * CAP_SYS_NICE (setscheduler)
 	 * CAP_IPC_LOCK (mlockall)
 	 */
-	aisexec_priv_drop ();
+	priv_drop ();
 
-	aisexec_mempool_init ();
+	corosync_mempool_init ();
 
-	corosync_ipc_init (
-		serialize_mutex_lock,
-		serialize_mutex_unlock,
-		gid_valid);
+	ipc_subsys_id = _logsys_subsys_create ("IPC", LOG_INFO);
+
+	coroipcs_ipc_init (&ipc_init_state);
 
 	/*
 	 * Start main processing loop
 	 */
-	poll_run (aisexec_poll_handle);
+	poll_run (corosync_poll_handle);
 
 	return EXIT_SUCCESS;
 }
