@@ -34,6 +34,8 @@
 
 /* IPv4/6 abstraction */
 
+#include <config.h>
+
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -124,8 +126,8 @@ void totemip_copy_endian_convert(struct totem_ip_address *addr1, struct totem_ip
 int totemip_compare(const void *a, const void *b)
 {
 	int i;
-	struct totem_ip_address *totemip_a = (struct totem_ip_address *)a;
-	struct totem_ip_address *totemip_b = (struct totem_ip_address *)b;
+	const struct totem_ip_address *totemip_a = (const struct totem_ip_address *)a;
+	const struct totem_ip_address *totemip_b = (const struct totem_ip_address *)b;
 	struct in_addr ipv4_a1;
 	struct in_addr ipv4_a2;
 	struct in6_addr ipv6_a1;
@@ -175,7 +177,7 @@ int totemip_compare(const void *a, const void *b)
 /* Build a localhost totem_ip_address */
 int totemip_localhost(int family, struct totem_ip_address *localhost)
 {
-	char *addr_text;
+	const char *addr_text;
 
 	memset (localhost, 0, sizeof (struct totem_ip_address));
 
@@ -205,11 +207,11 @@ int totemip_localhost_check(struct totem_ip_address *addr)
 	return totemip_equal(addr, &localhost);
 }
 
-char *totemip_print(struct totem_ip_address *addr)
+const char *totemip_print(struct totem_ip_address *addr)
 {
 	static char buf[INET6_ADDRSTRLEN];
 
-	return ((char *)inet_ntop(addr->family, addr->addr, buf, sizeof(buf)));
+	return (inet_ntop(addr->family, addr->addr, buf, sizeof(buf)));
 }
 
 /* Make a totem_ip_address into a usable sockaddr_storage */
@@ -226,7 +228,7 @@ int totemip_totemip_to_sockaddr_convert(struct totem_ip_address *ip_addr,
 		sin->sin_len = sizeof(struct sockaddr_in);
 #endif
 		sin->sin_family = ip_addr->family;
-		sin->sin_port = port;
+		sin->sin_port = ntohs(port);
 		memcpy(&sin->sin_addr, ip_addr->addr, sizeof(struct in_addr));
 		*addrlen = sizeof(struct sockaddr_in);
 		ret = 0;
@@ -240,7 +242,7 @@ int totemip_totemip_to_sockaddr_convert(struct totem_ip_address *ip_addr,
 		sin->sin6_len = sizeof(struct sockaddr_in6);
 #endif
 		sin->sin6_family = ip_addr->family;
-		sin->sin6_port = port;
+		sin->sin6_port = ntohs(port);
 		sin->sin6_scope_id = 2;
 		memcpy(&sin->sin6_addr, ip_addr->addr, sizeof(struct in6_addr));
 
@@ -313,7 +315,8 @@ int totemip_sockaddr_to_totemip_convert(struct sockaddr_storage *saddr, struct t
 int totemip_iface_check(struct totem_ip_address *bindnet,
 			struct totem_ip_address *boundto,
 			int *interface_up,
-			int *interface_num)
+			int *interface_num,
+			int mask_high_bit)
 {
 #define NEXT_IFR(a)	((struct ifreq *)((u_char *)&(a)->ifr_addr +\
 	((a)->ifr_addr.sa_len ? (a)->ifr_addr.sa_len : sizeof((a)->ifr_addr))))
@@ -376,7 +379,8 @@ static void parse_rtattr(struct rtattr *tb[], int max, struct rtattr *rta, int l
 int totemip_iface_check(struct totem_ip_address *bindnet,
 			struct totem_ip_address *boundto,
 			int *interface_up,
-			int *interface_num)
+			int *interface_num,
+			int mask_high_bit)
 {
 	int fd;
 	struct {
@@ -405,6 +409,7 @@ int totemip_iface_check(struct totem_ip_address *bindnet,
         memset(&nladdr, 0, sizeof(nladdr));
         nladdr.nl_family = AF_NETLINK;
 
+        memset(&req, 0, sizeof(req));
         req.nlh.nlmsg_len = sizeof(req);
         req.nlh.nlmsg_type = RTM_GETADDR;
         req.nlh.nlmsg_flags = NLM_F_ROOT|NLM_F_MATCH|NLM_F_REQUEST;
@@ -472,7 +477,7 @@ int totemip_iface_check(struct totem_ip_address *bindnet,
 					memcpy(&network, RTA_DATA(tb[IFA_BROADCAST]), sizeof(uint32_t));
 					memcpy(&addr, bindnet->addr, sizeof(uint32_t));
 
-					if (addr == (network & netmask)) {
+					if ((addr & netmask) == (network & netmask)) {
 						memcpy(ipaddr.addr, RTA_DATA(tb[IFA_ADDRESS]), TOTEMIP_ADDRLEN);
 						found_if = 1;
 					}
@@ -514,6 +519,18 @@ int totemip_iface_check(struct totem_ip_address *bindnet,
 		}
 	}
 finished:
+	/*
+	 * Mask 32nd bit off to workaround bugs in other poeples code
+	 * if configuration requests it.
+	 */
+	if (ipaddr.family == AF_INET && ipaddr.nodeid == 0) {
+                unsigned int nodeid = 0;
+                memcpy (&nodeid, ipaddr.addr, sizeof (int));
+		if (mask_high_bit) {
+                        nodeid &= 0x7FFFFFFF;
+		}
+                ipaddr.nodeid = nodeid;
+        }
 	totemip_copy (boundto, &ipaddr);
 	close(fd);
 	return 0;
