@@ -33,6 +33,8 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <config.h>
+
 #include <assert.h>
 #include <pthread.h>
 #include <sys/mman.h>
@@ -103,7 +105,7 @@ struct totemnet_instance {
 
 	unsigned int totemnet_private_key_len;
 
-	poll_handle totemnet_poll_handle;
+	hdb_handle_t totemnet_poll_handle;
 
 	struct totem_interface *totem_interface;
 
@@ -137,9 +139,16 @@ struct totemnet_instance {
 
 	int totemnet_log_level_debug;
 
-	void (*totemnet_log_printf) (char *file, int line, int level, char *format, ...) __attribute__((format(printf, 4, 5)));
+	int totemnet_subsys_id;
 
-	totemnet_handle handle;
+	void (*totemnet_log_printf) (int subsys,
+		const char *function,
+		const char *file,
+		int line, unsigned int level,
+		const char *format,
+		...)__attribute__((format(printf, 6, 7)));
+
+	hdb_handle_t handle;
 
 	char iov_buffer[FRAME_SIZE_MAX];
 
@@ -226,8 +235,12 @@ static void totemnet_instance_initialize (struct totemnet_instance *instance)
 	instance->my_memb_entries = 1;
 }
 
-#define log_printf(level, format, args...) \
-    instance->totemnet_log_printf (__FILE__, __LINE__, level, format, ##args)
+#define log_printf(level, format, args...)				\
+do {									\
+        instance->totemnet_log_printf (instance->totemnet_subsys_id,	\
+                __FUNCTION__, __FILE__, __LINE__,			\
+		level, (const char *)format, ##args);			\
+} while (0);
 
 static int authenticate_and_decrypt (
 	struct totemnet_instance *instance,
@@ -267,7 +280,7 @@ static int authenticate_and_decrypt (
 	hmac_init (&instance->totemnet_hmac_state, DIGEST_SHA1, hmac_key, 16);
 
 	hmac_process (&instance->totemnet_hmac_state, 
-		iov->iov_base + HMAC_HASH_SIZE,
+		(unsigned char *)iov->iov_base + HMAC_HASH_SIZE,
 		iov->iov_len - HMAC_HASH_SIZE);
 
 	len = hash_descriptor[DIGEST_SHA1]->hashsize;
@@ -282,7 +295,8 @@ static int authenticate_and_decrypt (
 	/*
 	 * Decrypt the contents of the message with the cipher key
 	 */
-	sober128_read (iov->iov_base + sizeof (struct security_header),
+	sober128_read ((unsigned char*)iov->iov_base +
+			sizeof (struct security_header),
 		iov->iov_len - sizeof (struct security_header),
 		&stream_prng_state);
 
@@ -575,7 +589,7 @@ static void totemnet_mcast_worker_fn (void *thread_state, void *work_item_in)
 }
 
 int totemnet_finalize (
-	totemnet_handle handle)
+	hdb_handle_t handle)
 {
 	struct totemnet_instance *instance;
 	int res = 0;
@@ -600,7 +614,7 @@ error_exit:
  */
 
 static int net_deliver_fn (
-	poll_handle handle,
+	hdb_handle_t handle,
 	int fd,
 	int revents,
 	void *data)
@@ -661,7 +675,7 @@ static int net_deliver_fn (
 			iovec->iov_len = FRAME_SIZE_MAX;
 			return 0;
 		}
-		msg_offset = iovec->iov_base +
+		msg_offset = (unsigned char *)iovec->iov_base +
 			sizeof (struct security_header);
 		size_delv = bytes_received - sizeof (struct security_header);
 	} else {
@@ -691,18 +705,9 @@ static int netif_determine (
 	int res;
 
 	res = totemip_iface_check (bindnet, bound_to,
-		interface_up, interface_num);
+		interface_up, interface_num,
++               0); // TODO andrew can address this instance->totem_config->clear_node_high_bit);
 
-	/*
-	 * If the desired binding is to an IPV4 network and nodeid isn't
-	 * specified, retrieve the node id from this_ip network address
-	 *
-	 * IPV6 networks must have a node ID specified since the node id
-	 * field is only 32 bits.
-	 */
-	if (bound_to->family == AF_INET && bound_to->nodeid == 0) {
-		memcpy (&bound_to->nodeid, bound_to->addr, sizeof (int));
-	}
 
 	return (res);
 }
@@ -1152,8 +1157,8 @@ static int totemnet_build_sockets (
  * Create an instance
  */
 int totemnet_initialize (
-	poll_handle poll_handle,
-	totemnet_handle *handle,
+	hdb_handle_t poll_handle,
+	hdb_handle_t *handle,
 	struct totem_config *totem_config,
 	int interface_no,
 	void *context,
@@ -1192,6 +1197,7 @@ int totemnet_initialize (
 	instance->totemnet_log_level_warning = totem_config->totem_logging_configuration.log_level_warning;
 	instance->totemnet_log_level_notice = totem_config->totem_logging_configuration.log_level_notice;
 	instance->totemnet_log_level_debug = totem_config->totem_logging_configuration.log_level_debug;
+	instance->totemnet_subsys_id = totem_config->totem_logging_configuration.log_subsys_id;
 	instance->totemnet_log_printf = totem_config->totem_logging_configuration.log_printf;
 
 	/*
@@ -1251,7 +1257,7 @@ error_destroy:
 }
 
 int totemnet_processor_count_set (
-	totemnet_handle handle,
+	hdb_handle_t handle,
 	int processor_count)
 {
 	struct totemnet_instance *instance;
@@ -1280,7 +1286,7 @@ error_exit:
 	return (res);
 }
 
-int totemnet_recv_flush (totemnet_handle handle)
+int totemnet_recv_flush (hdb_handle_t handle)
 {
 	struct totemnet_instance *instance;
 	struct pollfd ufd;
@@ -1314,7 +1320,7 @@ error_exit:
 	return (res);
 }
 
-int totemnet_send_flush (totemnet_handle handle)
+int totemnet_send_flush (hdb_handle_t handle)
 {
 	struct totemnet_instance *instance;
 	int res = 0;
@@ -1335,7 +1341,7 @@ error_exit:
 }
 
 int totemnet_token_send (
-	totemnet_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	int iov_len)
 {
@@ -1357,7 +1363,7 @@ error_exit:
 	return (res);
 }
 int totemnet_mcast_flush_send (
-	totemnet_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	unsigned int iov_len)
 {
@@ -1380,7 +1386,7 @@ error_exit:
 }
 
 int totemnet_mcast_noflush_send (
-	totemnet_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	unsigned int iov_len)
 {
@@ -1411,7 +1417,7 @@ error_exit:
 	return (res);
 }
 
-extern int totemnet_iface_check (totemnet_handle handle)
+extern int totemnet_iface_check (hdb_handle_t handle)
 {
 	struct totemnet_instance *instance;
 	int res = 0;
@@ -1441,10 +1447,10 @@ extern void totemnet_net_mtu_adjust (struct totem_config *totem_config)
 	}
 }
 
-char *totemnet_iface_print (totemnet_handle handle)  {
+const char *totemnet_iface_print (hdb_handle_t handle)  {
 	struct totemnet_instance *instance;
 	int res = 0;
-	char *ret_char;
+	const char *ret_char;
 
 	res = hdb_handle_get (&totemnet_instance_database, handle,
 		(void *)&instance);
@@ -1453,7 +1459,7 @@ char *totemnet_iface_print (totemnet_handle handle)  {
 		goto error_exit;
 	}
 	
-	ret_char = (char *)totemip_print (&instance->my_id);
+	ret_char = totemip_print (&instance->my_id);
 
 	hdb_handle_put (&totemnet_instance_database, handle);
 error_exit:
@@ -1461,7 +1467,7 @@ error_exit:
 }
 
 int totemnet_iface_get (
-	totemnet_handle handle,
+	hdb_handle_t handle,
 	struct totem_ip_address *addr)
 {
 	struct totemnet_instance *instance;
@@ -1482,7 +1488,7 @@ error_exit:
 }
 
 int totemnet_token_target_set (
-	totemnet_handle handle,
+	hdb_handle_t handle,
 	struct totem_ip_address *token_target)
 {
 	struct totemnet_instance *instance;
