@@ -82,6 +82,8 @@
  *	
  */
 
+#include <config.h>
+
 #include <netinet/in.h>
 #include <sys/uio.h>
 #include <stdio.h>
@@ -145,6 +147,8 @@ static unsigned short mcast_packed_msg_lens[FRAME_SIZE_MAX];
 
 static int mcast_packed_msg_count = 0;
 
+static int totempg_reserved = 0;
+
 /*
  * Function and data used to log messages
  */
@@ -153,7 +157,10 @@ static int totempg_log_level_error;
 static int totempg_log_level_warning;
 static int totempg_log_level_notice;
 static int totempg_log_level_debug;
-static void (*totempg_log_printf) (char *file, int line, int level, char *format, ...) __attribute__((format(printf, 4, 5))) = NULL;
+static int totempg_subsys_id;
+static void (*totempg_log_printf) (int subsys_id, const char *function,
+	const char *file, int line, unsigned int level,
+	const char *format, ...) __attribute__((format(printf, 6, 7)));
 
 struct totem_config *totempg_totem_config;
 
@@ -165,6 +172,18 @@ struct assembly {
 	struct list_head list;
 };
 
+static void assembly_deref (struct assembly *assembly);
+
+static int callback_token_received_fn (enum totem_callback_token_type type,
+	void *data);
+
+enum throw_away_mode_t {
+	THROW_AWAY_INACTIVE,
+	THROW_AWAY_ACTIVE
+};
+
+static enum throw_away_mode_t throw_away_mode = THROW_AWAY_INACTIVE;
+
 DECLARE_LIST_INIT(assembly_list_inuse);
 
 DECLARE_LIST_INIT(assembly_list_free);
@@ -215,8 +234,6 @@ static struct hdb_handle_database totempg_groups_instance_database = {
 	.mutex		= PTHREAD_MUTEX_INITIALIZER
 };
 
-static int send_ok (int msg_size);
-
 static unsigned char next_fragment = 1;
 
 static pthread_mutex_t totempg_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -225,8 +242,15 @@ static pthread_mutex_t callback_token_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static pthread_mutex_t mcast_msg_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-#define log_printf(level, format, args...) \
-    totempg_log_printf (__FILE__, __LINE__, level, format, ##args)
+#define log_printf(level, format, args...)				\
+do {									\
+        totempg_log_printf (totempg_subsys_id, __FUNCTION__,		\
+		__FILE__, __LINE__, level, format, ##args);		\
+} while (0);
+
+static int msg_count_send_ok (int msg_count);
+
+static int byte_count_send_ok (int byte_count);
 
 static struct assembly *assembly_ref (unsigned int nodeid)
 {
@@ -274,7 +298,7 @@ static struct assembly *assembly_ref (unsigned int nodeid)
 	return (assembly);
 }
 
-void assembly_deref (struct assembly *assembly)
+static void assembly_deref (struct assembly *assembly)
 {
 	list_del (&assembly->list);
 	list_add (&assembly->list, &assembly_list_free);
@@ -293,7 +317,7 @@ static inline void app_confchg_fn (
 
 	for (i = 0; i <= totempg_max_handle; i++) {
 		res = hdb_handle_get (&totempg_groups_instance_database,
-			i, (void *)&instance);
+			hdb_nocheck_convert (i), (void *)&instance);
 
 		if (res == 0) {
 			if (instance->confchg_fn) {
@@ -308,7 +332,8 @@ static inline void app_confchg_fn (
 					ring_id);
 			}
 
-			hdb_handle_put (&totempg_groups_instance_database, i);
+			hdb_handle_put (&totempg_groups_instance_database,
+				hdb_nocheck_convert (i));
 		}
 	}
 }
@@ -318,13 +343,30 @@ static inline void group_endian_convert (
 {
 	unsigned short *group_len;
 	int i;
+	struct iovec iovec_aligned = { NULL, 0 };
+	struct iovec *iovec_swab;
 
-	group_len = (unsigned short *)iovec->iov_base;
+	/*
+	 * Align data structure for sparc and ia64
+	 */
+	if ((size_t)iovec->iov_base % 4 != 0) {
+		iovec_aligned.iov_base = alloca(iovec->iov_len);
+		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);
+		iovec_aligned.iov_len = iovec->iov_len;
+		iovec_swab = &iovec_aligned;
+	} else {
+		iovec_swab = iovec;
+	}
+
+	group_len = (unsigned short *)iovec_swab->iov_base;
 	group_len[0] = swab16(group_len[0]);
 	for (i = 1; i < group_len[0] + 1; i++) {
 		group_len[i] = swab16(group_len[i]);
 	}
 
+	if (iovec_swab == &iovec_aligned) {
+		memcpy(iovec->iov_base, iovec_aligned.iov_base, iovec->iov_len);
+	}
 }
 
 static inline int group_matches (
@@ -347,7 +389,8 @@ static inline int group_matches (
 	 */
 	if ((size_t)iovec->iov_base % 4 != 0) {
 		iovec_aligned.iov_base = alloca(iovec->iov_len);
-		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);                iovec_aligned.iov_len = iovec->iov_len;
+		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);
+		iovec_aligned.iov_len = iovec->iov_len;
 		iovec = &iovec_aligned;
 	}
 
@@ -408,7 +451,7 @@ static inline void app_deliver_fn (
 
 	for (i = 0; i <= totempg_max_handle; i++) {
 		res = hdb_handle_get (&totempg_groups_instance_database,
-			i, (void *)&instance);
+			hdb_nocheck_convert (i), (void *)&instance);
 
 		if (res == 0) {
 			assert (iov_len == 1);
@@ -419,14 +462,14 @@ static inline void app_deliver_fn (
 				/*
 				 * Align data structure for sparc and ia64
 				 */
-				if (iovec->iov_base + adjust_iovec % 4 != 0) {
+				if ((char *)iovec->iov_base + adjust_iovec % 4 != 0) {
 					/*
 					 * Deal with misalignment
 					 */
 					stripped_iovec.iov_base =
 						alloca (stripped_iovec.iov_len);
 					memcpy (stripped_iovec.iov_base,
-						 iovec->iov_base + adjust_iovec,
+						 (char *)iovec->iov_base + adjust_iovec,
 						stripped_iovec.iov_len);
 				}
 
@@ -437,7 +480,7 @@ static inline void app_deliver_fn (
 					endian_conversion_required);
 			}
 
-			hdb_handle_put (&totempg_groups_instance_database, i);
+			hdb_handle_put (&totempg_groups_instance_database, hdb_nocheck_convert(i));
 		}
 	}
 }
@@ -558,43 +601,32 @@ static void totempg_deliver_fn (
 	 * the continued message.
 	 */
 	start = 0;
-	if (continuation) {
 
-		if (continuation != assembly->last_frag_num) {
-			log_printf (totempg_log_level_error,
-				"Message continuation doesn't match previous frag e: %u - a: %u\n",
-				assembly->last_frag_num, continuation);
-			continuation = 0;
-		}
+	if (throw_away_mode == THROW_AWAY_ACTIVE) {
+		 /* Throw away the first msg block */
+		if (mcast->fragmented == 0 || mcast->fragmented == 1) {
+			throw_away_mode = THROW_AWAY_INACTIVE;
 
-		if ((assembly->index == 0) ||
-							(!continuation && assembly->index)) {
-			log_printf (totempg_log_level_error,
-				"Throwing away broken message: continuation %u, index %u\n",
-				continuation, assembly->index);
-			continuation = 0;
-		}
-		
-		/* 
-		 * we decided to throw away the first continued message
-		 * in this buffer, if continuation was set to zero.
-		 */
-		if (!continuation) {
 			assembly->index += msg_lens[0];
 			iov_delv.iov_base = &assembly->data[assembly->index];
 			iov_delv.iov_len = msg_lens[1];
 			start = 1;
 		}
-			
-	}
-
-	for  (i = start; i < msg_count; i++) {
-		app_deliver_fn(nodeid, &iov_delv, 1,
-			endian_conversion_required);
-		assembly->index += msg_lens[i];
-		iov_delv.iov_base = &assembly->data[assembly->index];
-		if (i < (msg_count - 1)) {
-			iov_delv.iov_len = msg_lens[i + 1];
+	} else 
+	if (throw_away_mode == THROW_AWAY_INACTIVE) {
+		if (continuation == assembly->last_frag_num) {
+			assembly->last_frag_num = mcast->fragmented;
+			for  (i = start; i < msg_count; i++) {
+				app_deliver_fn(nodeid, &iov_delv, 1,
+					endian_conversion_required);
+				assembly->index += msg_lens[i];
+				iov_delv.iov_base = &assembly->data[assembly->index];
+				if (i < (msg_count - 1)) {
+					iov_delv.iov_len = msg_lens[i + 1];
+				}
+			}
+		} else {
+			throw_away_mode = THROW_AWAY_ACTIVE;
 		}
 	}
 
@@ -609,7 +641,6 @@ static void totempg_deliver_fn (
 		/*
 		 * Message is fragmented, keep around assembly list
 		 */
-		assembly->last_frag_num = mcast->fragmented;
 		if (mcast->msg_count > 1) {
 			memmove (&assembly->data[0],
 				&assembly->data[assembly->index],
@@ -674,7 +705,7 @@ int callback_token_received_fn (enum totem_callback_token_type type,
  * Initialize the totem process group abstraction
  */
 int totempg_initialize (
-	poll_handle poll_handle,
+	hdb_handle_t poll_handle,
 	struct totem_config *totem_config)
 {
 	int res;
@@ -686,6 +717,7 @@ int totempg_initialize (
 	totempg_log_level_notice = totem_config->totem_logging_configuration.log_level_notice;
 	totempg_log_level_debug = totem_config->totem_logging_configuration.log_level_debug;
 	totempg_log_printf = totem_config->totem_logging_configuration.log_printf;
+	totempg_subsys_id = totem_config->totem_logging_configuration.log_subsys_id;
 
 	fragmentation_data = malloc (TOTEMPG_PACKET_SIZE);
 	if (fragmentation_data == 0) {
@@ -721,14 +753,16 @@ void totempg_finalize (void)
  * Multicast a message
  */
 static int mcast_msg (
-	struct iovec *iovec,
+	struct iovec *iovec_in,
 	int iov_len,
 	int guarantee)
 {
 	int res = 0;
 	struct totempg_mcast mcast;
 	struct iovec iovecs[3];
+	struct iovec iovec[64];
 	int i;
+	int dest, src;
 	int max_packet_size = 0;
 	int copy_len = 0; 
 	int copy_base = 0;
@@ -737,6 +771,18 @@ static int mcast_msg (
 	pthread_mutex_lock (&mcast_msg_mutex);
 	totemmrp_new_msg_signal ();
 
+	/*
+	 * Remove zero length iovectors from the list
+	 */
+	assert (iov_len < 64);
+	for (dest = 0, src = 0; src < iov_len; src++) {
+		if (iovec_in[src].iov_len) {
+			memcpy (&iovec[dest++], &iovec_in[src],
+				sizeof (struct iovec));
+		}
+	}
+	iov_len = dest;
+
 	max_packet_size = TOTEMPG_PACKET_SIZE -
 		(sizeof (unsigned short) * (mcast_packed_msg_count + 1));
 
@@ -749,7 +795,7 @@ static int mcast_msg (
 		total_size += iovec[i].iov_len;
 	}
 
-	if (send_ok (total_size + sizeof(unsigned short) *
+	if (byte_count_send_ok (total_size + sizeof(unsigned short) *
 		(mcast_packed_msg_count+1)) == 0) {
 
 		pthread_mutex_unlock (&mcast_msg_mutex);
@@ -771,9 +817,10 @@ static int mcast_msg (
 			(max_packet_size - sizeof (unsigned short))) {
 
 			memcpy (&fragmentation_data[fragment_size],
-				iovec[i].iov_base + copy_base, copy_len);
+				(char *)iovec[i].iov_base + copy_base, copy_len);
 			fragment_size += copy_len;
 			mcast_packed_msg_lens[mcast_packed_msg_count] += copy_len;
+			next_fragment = 1;
 			copy_len = 0;
 			copy_base = 0;
 			i++;
@@ -787,15 +834,15 @@ static int mcast_msg (
 
 			copy_len = min(copy_len, max_packet_size - fragment_size);
 			if( copy_len == max_packet_size )
-				data_ptr = iovec[i].iov_base + copy_base;
+				data_ptr = (unsigned char *)iovec[i].iov_base + copy_base;
 			else {
 				data_ptr = fragmentation_data;
 				memcpy (&fragmentation_data[fragment_size],
-				iovec[i].iov_base + copy_base, copy_len);
+				(unsigned char *)iovec[i].iov_base + copy_base, copy_len);
 			}
 
 			memcpy (&fragmentation_data[fragment_size],
-				iovec[i].iov_base + copy_base, copy_len);
+				(unsigned char *)iovec[i].iov_base + copy_base, copy_len);
 			mcast_packed_msg_lens[mcast_packed_msg_count] += copy_len;
 
 			/*
@@ -871,23 +918,44 @@ static int mcast_msg (
 /*
  * Determine if a message of msg_size could be queued
  */
-#define FUZZY_AVAIL_SUBTRACT 5
-static int send_ok (
-	int msg_size)
+static int msg_count_send_ok (
+	int msg_count)
 {
 	int avail = 0;
-	int total;
 
-	avail = totemmrp_avail () - FUZZY_AVAIL_SUBTRACT;
+	avail = totemmrp_avail () - totempg_reserved - 1;
 	
-	/*
-	 * msg size less then totempg_totem_config->net_mtu - 25 will take up
-	 * a full message, so add +1
-	 * totempg_totem_config->net_mtu - 25 is for the totempg_mcast header
-	 */
-	total = (msg_size / (totempg_totem_config->net_mtu - 25)) + 1; 
+	return (avail > msg_count);
+}
 
-	return (avail >= total);
+static int byte_count_send_ok (
+	int byte_count)
+{
+	unsigned int msg_count = 0;
+	int avail = 0;
+
+	avail = totemmrp_avail () - 1;
+
+	msg_count = (byte_count / (totempg_totem_config->net_mtu - 25)) + 1; 
+
+	return (avail > msg_count);
+}
+
+static int send_reserve (
+	int msg_size)
+{
+	unsigned int msg_count = 0;
+
+	msg_count = (msg_size / (totempg_totem_config->net_mtu - 25)) + 1; 
+	totempg_reserved += msg_count;
+
+	return (msg_count);
+}
+
+static void send_release (
+	int msg_count)
+{
+	totempg_reserved -= msg_count;
 }
 
 int totempg_callback_token_create (
@@ -918,7 +986,7 @@ void totempg_callback_token_destroy (
  */
 
 int totempg_groups_initialize (
-	totempg_groups_handle *handle,
+	hdb_handle_t *handle,
 
 	void (*deliver_fn) (
 		unsigned int nodeid,
@@ -972,7 +1040,7 @@ error_exit:
 }
 
 int totempg_groups_join (
-	totempg_groups_handle handle,
+	hdb_handle_t handle,
 	struct totempg_group *groups,
 	int group_cnt)
 {
@@ -1007,7 +1075,7 @@ error_exit:
 }
 
 int totempg_groups_leave (
-	totempg_groups_handle handle,
+	hdb_handle_t handle,
 	struct totempg_group *groups,
 	int group_cnt)
 {
@@ -1032,7 +1100,7 @@ error_exit:
 #define MAX_GROUPS_PER_MSG 32
 
 int totempg_groups_mcast_joined (
-	totempg_groups_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	int iov_len,
 	int guarantee)
@@ -1074,8 +1142,8 @@ error_exit:
 	return (res);
 }
 
-int totempg_groups_send_ok_joined (
-	totempg_groups_handle handle,
+int totempg_groups_joined_reserve (
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	int iov_len)
 {
@@ -1083,6 +1151,7 @@ int totempg_groups_send_ok_joined (
 	unsigned int size = 0;
 	unsigned int i;
 	unsigned int res;
+	unsigned int reserved = 0;
 
 	pthread_mutex_lock (&totempg_mutex);
 	pthread_mutex_lock (&mcast_msg_mutex);
@@ -1099,18 +1168,32 @@ int totempg_groups_send_ok_joined (
 		size += iovec[i].iov_len;
 	}
 
-	res = send_ok (size);
+	reserved = send_reserve (size);
+	if (msg_count_send_ok (reserved) == 0) {
+		send_release (reserved);
+		reserved = 0;
+	}
 
 	hdb_handle_put (&totempg_groups_instance_database, handle);
 
 error_exit:
 	pthread_mutex_unlock (&mcast_msg_mutex);
 	pthread_mutex_unlock (&totempg_mutex);
-	return (res);
+	return (reserved);
+}
+
+
+void totempg_groups_joined_release (int msg_count)
+{
+	pthread_mutex_lock (&totempg_mutex);
+	pthread_mutex_lock (&mcast_msg_mutex);
+	send_release (msg_count);
+	pthread_mutex_unlock (&mcast_msg_mutex);
+	pthread_mutex_unlock (&totempg_mutex);
 }
 
 int totempg_groups_mcast_groups (
-	totempg_groups_handle handle,
+	hdb_handle_t handle,
 	int guarantee,
 	struct totempg_group *groups,
 	int groups_cnt,
@@ -1159,7 +1242,7 @@ error_exit:
  * Returns -1 if error, 0 if can't send, 1 if can send the message
  */
 int totempg_groups_send_ok_groups (
-	totempg_groups_handle handle,
+	hdb_handle_t handle,
 	struct totempg_group *groups,
 	int groups_cnt,
 	struct iovec *iovec,
@@ -1184,7 +1267,7 @@ int totempg_groups_send_ok_groups (
 		size += iovec[i].iov_len;
 	}
 
-	res = send_ok (size);
+	res = msg_count_send_ok (size);
 	 
 	hdb_handle_put (&totempg_groups_instance_database, handle);
 error_exit:
@@ -1218,7 +1301,7 @@ int totempg_ring_reenable (void)
 	return (res);
 }
 
-char *totempg_ifaces_print (unsigned int nodeid)
+const char *totempg_ifaces_print (unsigned int nodeid)
 {
 	static char iface_string[256 * INTERFACE_MAX];
 	char one_iface[64];
@@ -1243,7 +1326,7 @@ char *totempg_ifaces_print (unsigned int nodeid)
 	return (iface_string);
 }
 
-int totempg_my_nodeid_get (void)
+unsigned int totempg_my_nodeid_get (void)
 {
 	return (totemmrp_my_nodeid_get());
 }
