@@ -33,6 +33,8 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <config.h>
+
 #include <assert.h>
 #include <pthread.h>
 #include <sys/mman.h>
@@ -66,6 +68,15 @@
 #include "totemnet.h"
 #include "totemrrp.h"
 
+void rrp_deliver_fn (
+	void *context,
+	void *msg,
+	int msg_len);
+
+void rrp_iface_change_fn (
+	void *context,
+	struct totem_ip_address *iface_addr);
+
 struct totemrrp_instance;
 struct passive_instance {
 	struct totemrrp_instance *rrp_instance;
@@ -95,7 +106,7 @@ struct active_instance {
 };
 
 struct rrp_algo {
-	char *name;
+	const char *name;
 
 	void * (*initialize) (
 		struct totemrrp_instance *rrp_instance,
@@ -154,7 +165,7 @@ struct rrp_algo {
 };
 
 struct totemrrp_instance {
-	poll_handle totemrrp_poll_handle;
+	hdb_handle_t totemrrp_poll_handle;
 
 	struct totem_interface *interfaces;
 
@@ -194,17 +205,21 @@ struct totemrrp_instance {
 
 	int totemrrp_log_level_debug;
 
-	void (*totemrrp_log_printf) (char *file, int line, int level, char *format, ...) __attribute__((format(printf, 4, 5)));
+	int totemrrp_subsys_id;
 
-	totemrrp_handle handle;
+	void (*totemrrp_log_printf) (int subsys, const char *function,
+		const char *file, int line, unsigned int level,
+		const char *format, ...)__attribute__((format(printf, 6, 7)));
 
-	totemnet_handle *net_handles;
+	hdb_handle_t handle;
+
+	hdb_handle_t *net_handles;
 
 	void *rrp_algo_instance;
 
 	int interface_count;
 
-	int poll_handle;
+	hdb_handle_t poll_handle;
 
 	int processor_count;
 
@@ -459,8 +474,14 @@ static struct hdb_handle_database totemrrp_instance_database = {
 	.mutex		= PTHREAD_MUTEX_INITIALIZER
 };
 
-#define log_printf(level, format, args...) \
-    rrp_instance->totemrrp_log_printf (__FILE__, __LINE__, level, format, ##args)
+
+#define log_printf(level, format, args...)				\
+do {									\
+	rrp_instance->totemrrp_log_printf (				\
+		rrp_instance->totemrrp_subsys_id,			\
+		__FUNCTION__, __FILE__, __LINE__, level,		\
+		format, ##args);					\
+} while (0);
 
 /*
  * None Replication Implementation
@@ -717,6 +738,7 @@ static void passive_mcast_recv (
 				totemnet_iface_print (rrp_instance->net_handles[i]));
 			log_printf (
 				rrp_instance->totemrrp_log_level_error,
+				"%s",
 				rrp_instance->status[i]);
 		}
 	}
@@ -800,6 +822,7 @@ static void passive_token_recv (
 				totemnet_iface_print (rrp_instance->net_handles[i]));
 			log_printf (
 				rrp_instance->totemrrp_log_level_error,
+				"%s",
 				rrp_instance->status[i]);
 		}
 	}
@@ -972,6 +995,7 @@ static void timer_function_active_problem_decrementer (void *context)
 			}
 				log_printf (
 					rrp_instance->totemrrp_log_level_warning,
+					"%s",
 					rrp_instance->status[i]);
 		}
 	}
@@ -1003,6 +1027,7 @@ static void timer_function_active_token_expired (void *context)
 				rrp_instance->totem_config->rrp_problem_count_threshold);
 			log_printf (
 				rrp_instance->totemrrp_log_level_warning,
+				"%s",
 				rrp_instance->status[i]);
 		}
 	}
@@ -1017,6 +1042,7 @@ static void timer_function_active_token_expired (void *context)
 				totemnet_iface_print (rrp_instance->net_handles[i]));
 			log_printf (
 				rrp_instance->totemrrp_log_level_error,
+				"%s",
 				rrp_instance->status[i]);
 			active_timer_problem_decrementer_cancel (active_instance);
 		}
@@ -1338,7 +1364,7 @@ void rrp_iface_change_fn (
 }
 
 int totemrrp_finalize (
-	totemrrp_handle handle)
+	hdb_handle_t handle)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1370,8 +1396,8 @@ error_exit:
  * Create an instance
  */
 int totemrrp_initialize (
-	poll_handle poll_handle,
-	totemrrp_handle *handle,
+	hdb_handle_t poll_handle,
+	hdb_handle_t *handle,
 	struct totem_config *totem_config,
 	void *context,
 
@@ -1426,6 +1452,7 @@ int totemrrp_initialize (
 	instance->totemrrp_log_level_warning = totem_config->totem_logging_configuration.log_level_warning;
 	instance->totemrrp_log_level_notice = totem_config->totem_logging_configuration.log_level_notice;
 	instance->totemrrp_log_level_debug = totem_config->totem_logging_configuration.log_level_debug;
+	instance->totemrrp_subsys_id = totem_config->totem_logging_configuration.log_subsys_id;
 	instance->totemrrp_log_printf = totem_config->totem_logging_configuration.log_printf;
 
 	instance->interfaces = totem_config->interfaces;
@@ -1442,7 +1469,7 @@ int totemrrp_initialize (
 
 	instance->interface_count = totem_config->interface_count;
 
-	instance->net_handles = malloc (sizeof (totemnet_handle) * totem_config->interface_count);
+	instance->net_handles = malloc (sizeof (hdb_handle_t) * totem_config->interface_count);
 
 	instance->context = context;
 
@@ -1481,7 +1508,7 @@ error_destroy:
 }
 
 int totemrrp_processor_count_set (
-	totemrrp_handle handle,
+	hdb_handle_t handle,
 	unsigned int processor_count)
 {
 	struct totemrrp_instance *instance;
@@ -1505,7 +1532,7 @@ error_exit:
 }
 
 int totemrrp_token_target_set (
-	totemrrp_handle handle,
+	hdb_handle_t handle,
 	struct totem_ip_address *addr,
 	unsigned int iface_no)
 {
@@ -1526,7 +1553,7 @@ int totemrrp_token_target_set (
 error_exit:
 	return (res);
 }
-int totemrrp_recv_flush (totemrrp_handle handle)
+int totemrrp_recv_flush (hdb_handle_t handle)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1546,7 +1573,7 @@ error_exit:
 	return (res);
 }
 
-int totemrrp_send_flush (totemrrp_handle handle)
+int totemrrp_send_flush (hdb_handle_t handle)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1567,7 +1594,7 @@ error_exit:
 }
 
 int totemrrp_token_send (
-	totemrrp_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	unsigned int iov_len)
 {
@@ -1590,7 +1617,7 @@ error_exit:
 }
 
 int totemrrp_mcast_flush_send (
-	totemrrp_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	unsigned int iov_len)
 {
@@ -1613,7 +1640,7 @@ error_exit:
 }
 
 int totemrrp_mcast_noflush_send (
-	totemrrp_handle handle,
+	hdb_handle_t handle,
 	struct iovec *iovec,
 	unsigned int iov_len)
 {
@@ -1643,7 +1670,7 @@ error_exit:
 	return (res);
 }
 
-int totemrrp_iface_check (totemrrp_handle handle)
+int totemrrp_iface_check (hdb_handle_t handle)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1663,7 +1690,7 @@ error_exit:
 }
 
 int totemrrp_ifaces_get (
-        totemrrp_handle handle,
+        hdb_handle_t handle,
 	char ***status,
 	unsigned int *iface_count)
 {
@@ -1690,12 +1717,13 @@ error_exit:
 }
 
 int totemrrp_ring_reenable (
-        totemrrp_handle handle)
+        hdb_handle_t handle)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
 	unsigned int i;
 
+printf ("totemrrp ring reenable\n");
 	res = hdb_handle_get (&totemrrp_instance_database, handle,
 		(void *)&instance);
 	if (res != 0) {
