@@ -38,11 +38,10 @@
 #ifdef COROSYNC_BSD
 #include <sys/uio.h>
 #endif
+#include <corosync/hdb.h>
 
 typedef void * corosync_timer_handle_t;
 
-typedef unsigned int corosync_tpg_handle;
-
 struct corosync_tpg_group {
 	void *group;
 	int group_len;
@@ -88,17 +87,36 @@ enum totem_configuration_type {
 };
 #endif
 
-enum corosync_lib_flow_control {
-	COROSYNC_LIB_FLOW_CONTROL_REQUIRED = 1,
-	COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED = 2
+#if !defined(TOTEM_CALLBACK_TOKEN_TYPE)
+enum totem_callback_token_type {
+	TOTEM_CALLBACK_TOKEN_RECEIVED = 1,
+	TOTEM_CALLBACK_TOKEN_SENT = 2
+};
+#endif
+
+enum cs_lib_flow_control {
+	CS_LIB_FLOW_CONTROL_REQUIRED = 1,
+	CS_LIB_FLOW_CONTROL_NOT_REQUIRED = 2
+};
+#define corosync_lib_flow_control cs_lib_flow_control
+#define COROSYNC_LIB_FLOW_CONTROL_REQUIRED CS_LIB_FLOW_CONTROL_REQUIRED
+#define COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED CS_LIB_FLOW_CONTROL_NOT_REQUIRED
+
+enum cs_lib_allow_inquorate {
+	CS_LIB_DISALLOW_INQUORATE = 0, /* default */
+	CS_LIB_ALLOW_INQUORATE = 1
 };
 
 #if !defined (COROSYNC_FLOW_CONTROL_STATE)
-enum corosync_flow_control_state {
-	COROSYNC_FLOW_CONTROL_STATE_DISABLED,
-	COROSYNC_FLOW_CONTROL_STATE_ENABLED
+enum cs_flow_control_state {
+	CS_FLOW_CONTROL_STATE_DISABLED,
+	CS_FLOW_CONTROL_STATE_ENABLED
 };
-#endif
+#define corosync_flow_control_state cs_flow_control_state
+#define CS_FLOW_CONTROL_STATE_DISABLED CS_FLOW_CONTROL_STATE_DISABLED
+#define CS_FLOW_CONTROL_STATE_ENABLED CS_FLOW_CONTROL_STATE_ENABLED
+
+#endif /* COROSYNC_FLOW_CONTROL_STATE */
 
 typedef enum {
 	COROSYNC_FATAL_ERROR_EXIT = -1,
@@ -109,11 +127,12 @@ typedef enum {
 	COROSYNC_DYNAMICLOAD = -12,
 	COROSYNC_OUT_OF_MEMORY = -15,
 	COROSYNC_FATAL_ERR = -16
-} corosync_fatal_error_t;
+} cs_fatal_error_t;
+#define corosync_fatal_error_t cs_fatal_error_t;
 
 #ifndef OBJECT_PARENT_HANDLE
 
-#define OBJECT_PARENT_HANDLE 0
+#define OBJECT_PARENT_HANDLE 0xffffffff00000000ULL
 
 struct object_valid {
 	char *object_name;
@@ -125,6 +144,7 @@ struct object_key_valid {
 	int key_len;
 	int (*validate_callback) (void *key, int key_len, void *value, int value_len);
 };
+/* deprecated */
 
 typedef enum {
 	OBJECT_TRACK_DEPTH_ONE,
@@ -137,155 +157,188 @@ typedef enum {
 	OBJECT_KEY_DELETED
 } object_change_type_t;
 
-typedef void (*object_key_change_notify_fn_t)(object_change_type_t change_type,
-											  unsigned int parent_object_handle,
-											  unsigned int object_handle,
-											  void *object_name_pt, int object_name_len,
-											  void *key_name_pt, int key_len,
-											  void *key_value_pt, int key_value_len,
-											  void *priv_data_pt);
+typedef enum {
+        OBJDB_RELOAD_NOTIFY_START,
+        OBJDB_RELOAD_NOTIFY_END,
+	OBJDB_RELOAD_NOTIFY_FAILED
+} objdb_reload_notify_type_t;
 
-typedef void (*object_create_notify_fn_t) (unsigned int parent_object_handle,
-										   unsigned int object_handle,
-										   uint8_t *name_pt, int name_len,
-										   void *priv_data_pt);
+typedef void (*object_key_change_notify_fn_t)(
+	object_change_type_t change_type,
+	hdb_handle_t parent_object_handle,
+	hdb_handle_t object_handle,
+	void *object_name_pt, int object_name_len,
+	void *key_name_pt, int key_len,
+	void *key_value_pt, int key_value_len,
+	void *priv_data_pt);
 
-typedef void (*object_destroy_notify_fn_t) (unsigned int parent_object_handle,
-											uint8_t *name_pt, int name_len,
-											void *priv_data_pt);
-typedef void (*object_notify_callback_fn_t)(unsigned int object_handle,
-											void *key_name, int key_len,
-											void *value, int value_len,
-											object_change_type_t type,
-											void * priv_data_pt);
+typedef void (*object_create_notify_fn_t) (
+	hdb_handle_t parent_object_handle,
+	hdb_handle_t object_handle,
+	uint8_t *name_pt, int name_len,
+	void *priv_data_pt);
+
+typedef void (*object_destroy_notify_fn_t) (
+	hdb_handle_t parent_object_handle,
+	uint8_t *name_pt, int name_len,
+	void *priv_data_pt);
+
+typedef void (*object_notify_callback_fn_t)(
+	hdb_handle_t object_handle,
+	void *key_name, int key_len,
+	void *value, int value_len,
+	object_change_type_t type,
+	void * priv_data_pt);
+
+typedef void (*object_reload_notify_fn_t) (
+	objdb_reload_notify_type_t,
+	int flush,
+	void *priv_data_pt);
 
 #endif /* OBJECT_PARENT_HANDLE_DEFINED */
 
+#ifndef QUORUM_H_DEFINED
+typedef void (*quorum_callback_fn_t) (int quorate, void *context);
+
+struct quorum_callin_functions
+{
+	int (*quorate) (void);
+	int (*register_callback) (quorum_callback_fn_t callback_fn, void *context);
+	int (*unregister_callback) (quorum_callback_fn_t callback_fn, void *context);
+};
+
+typedef void (*sync_callback_fn_t) (
+	unsigned int *view_list,
+	int view_list_entries,
+	int primary_designated,
+	struct memb_ring_id *ring_id);
+
+#endif /* QUORUM_H_DEFINED */
+
+
 struct corosync_api_v1 {
 	/*
 	 * Object and configuration APIs
 	 */
 	int (*object_create) (
-		unsigned int parent_object_handle,
-		unsigned int *object_handle,
+		hdb_handle_t parent_object_handle,
+		hdb_handle_t *object_handle,
 		void *object_name, unsigned int object_name_len);
 
 	int (*object_priv_set) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *priv);
 
 	int (*object_key_create) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *key_name,
 		int key_len,
 		void *value,
 		int value_len);
 
 	int (*object_destroy) (
-		unsigned int object_handle);
+		hdb_handle_t object_handle);
 
 	int (*object_valid_set) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		struct object_valid *object_valid_list,
 		unsigned int object_valid_list_entries);
 
 	int (*object_key_valid_set) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		struct object_key_valid *object_key_valid_list,
 		unsigned int object_key_valid_list_entries);
 
 	int (*object_find_create) (
-		unsigned int parent_object_handle,
+		hdb_handle_t parent_object_handle,
 		void *object_name,
 		int object_name_len,
-		unsigned int *object_find_handle);
+		hdb_handle_t *object_find_handle);
 
 	int (*object_find_next) (
-		unsigned int object_find_handle,
-		unsigned int *object_handle);
+		hdb_handle_t object_find_handle,
+		hdb_handle_t *object_handle);
 
 	int (*object_find_destroy) (
-		unsigned int object_find_handle);
+		hdb_handle_t object_find_handle);
 
 	int (*object_key_get) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *key_name,
 		int key_len,
 		void **value,
 		int *value_len);
 
 	int (*object_priv_get) (
-		unsigned int jobject_handle,
+		hdb_handle_t jobject_handle,
 		void **priv);
 
 	int (*object_key_replace) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *key_name,
 		int key_len,
-		void *old_value,
-		int old_value_len,
 		void *new_value,
 		int new_value_len);
 
 	int (*object_key_delete) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *key_name,
-		int key_len,
-		void *value,
-		int value_len);
+		int key_len);
 
 	int (*object_iter_reset) (
-		unsigned int parent_object_handle);
+		hdb_handle_t parent_object_handle);
 
 	int (*object_iter) (
-		unsigned int parent_object_handle,
+		hdb_handle_t parent_object_handle,
 		void **object_name,
 		int *name_len,
-		unsigned int *object_handle);
+		hdb_handle_t *object_handle);
 
 	int (*object_key_iter_reset) (
-		unsigned int object_handle);
+		hdb_handle_t object_handle);
 
 	int (*object_key_iter) (
-		unsigned int parent_object_handle,
+		hdb_handle_t parent_object_handle,
 		void **key_name,
 		int *key_len,
 		void **value,
 		int *value_len);
 
 	int (*object_parent_get) (
-		unsigned int object_handle,
-		unsigned int *parent_handle);
+		hdb_handle_t object_handle,
+		hdb_handle_t *parent_handle);
 
 	int (*object_name_get) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		char *object_name,
 		int *object_name_len);
 
 	int (*object_dump) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		FILE *file);
 
 	int (*object_key_iter_from) (
-		unsigned int parent_object_handle,
-		unsigned int start_pos,
+		hdb_handle_t parent_object_handle,
+		hdb_handle_t start_pos,
 		void **key_name,
 		int *key_len,
 		void **value,
 		int *value_len);
 
 	int (*object_track_start) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		object_track_depth_t depth,
 		object_key_change_notify_fn_t key_change_notify_fn,
 		object_create_notify_fn_t object_create_notify_fn,
 		object_destroy_notify_fn_t object_destroy_notify_fn,
+		object_reload_notify_fn_t object_reload_notify_fn,
 		void * priv_data_pt);
 
 	void (*object_track_stop) (
 		object_key_change_notify_fn_t key_change_notify_fn,
 		object_create_notify_fn_t object_create_notify_fn,
 		object_destroy_notify_fn_t object_destroy_notify_fn,
+		object_reload_notify_fn_t object_reload_notify_fn,
 		void * priv_data_pt);
 
 	int (*object_write_config) (char **error_string);
@@ -294,13 +347,13 @@ struct corosync_api_v1 {
 				     char **error_string);
 
 	int (*object_key_increment) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *key_name,
 		int key_len,
 		unsigned int *value);
 
 	int (*object_key_decrement) (
-		unsigned int object_handle,
+		hdb_handle_t object_handle,
 		void *key_name,
 		int key_len,
 		unsigned int *value);
@@ -325,6 +378,9 @@ struct corosync_api_v1 {
 
 	unsigned long long (*timer_time_get) (void);
 
+	unsigned long long (*timer_expire_time_get) (
+		corosync_timer_handle_t timer_handle);
+
 	/*
 	 * IPC APIs
 	 */
@@ -336,35 +392,11 @@ struct corosync_api_v1 {
 
 	int (*ipc_response_send) (void *conn, void *msg, int mlen);
 
-	int (*ipc_response_no_fcc) (void *conn, void *msg, int mlen);
+	int (*ipc_response_iov_send) (void *conn, struct iovec *iov, int iov_len);
 
 	int (*ipc_dispatch_send) (void *conn, void *msg, int mlen);
 
-	/*
-	 * DEPRECATED
-	 */
-	int (*ipc_conn_send_response) (void *conn, void *msg, int mlen);
-
-	/*
-	 * DEPRECATED
-	 */
-	void *(*ipc_conn_partner_get) (void *conn);
-
-	void (*ipc_fc_create) (
-		void *conn,
-		unsigned int service,
-		char *id,
-		int id_len,
-		void (*flow_control_state_set_fn)
-			(void *context,
-				enum corosync_flow_control_state flow_control_state_set),
-		void *context);
-
-	void (*ipc_fc_destroy) (
-		void *conn,
-		unsigned int service,
-		unsigned char *id,
-		int id_len);
+	int (*ipc_dispatch_iov_send) (void *conn, struct iovec *iov, int iov_len);
 
 	void (*ipc_refcnt_inc) (void *conn);
 
@@ -373,7 +405,7 @@ struct corosync_api_v1 {
 	/*
 	 * Totem APIs
 	 */
-	int (*totem_nodeid_get) (void);
+	unsigned int (*totem_nodeid_get) (void);
 
 	int (*totem_family_get) (void);
 
@@ -381,8 +413,6 @@ struct corosync_api_v1 {
 
 	int (*totem_mcast) (struct iovec *iovec, int iov_len, unsigned int guarantee);
 
-	int (*totem_send_ok) (struct iovec *iovec, int iov_len);
-
 	int (*totem_ifaces_get) (
 		unsigned int nodeid,
 		struct totem_ip_address *interfaces,
@@ -393,12 +423,20 @@ struct corosync_api_v1 {
 
 	char *(*totem_ip_print) (struct totem_ip_address *addr);
 
+
+	int (*totem_callback_token_create) (
+		void **handle_out,
+		enum totem_callback_token_type type,
+		int delete,
+		int (*callback_fn) (enum totem_callback_token_type type, void *),
+		void *data);
+
 	/*
 	 * Totem open process groups API for those service engines
 	 * wanting their own groups
 	 */
 	int (*tpg_init) (
-		corosync_tpg_handle *handle,
+		hdb_handle_t *handle,
 
 		void (*deliver_fn) (
 			unsigned int nodeid,
@@ -414,58 +452,77 @@ struct corosync_api_v1 {
 			struct memb_ring_id *ring_id));
 
 	int (*tpg_exit) (
-       		corosync_tpg_handle handle);
+       		hdb_handle_t handle);
 
 	int (*tpg_join) (
-		corosync_tpg_handle handle,
+		hdb_handle_t handle,
 		struct corosync_tpg_group *groups,
 		int gruop_cnt);
 
 	int (*tpg_leave) (
-		corosync_tpg_handle handle,
+		hdb_handle_t handle,
 		struct corosync_tpg_group *groups,
 		int gruop_cnt);
 
 	int (*tpg_joined_mcast) (
-		corosync_tpg_handle handle,
+		hdb_handle_t handle,
 		struct iovec *iovec,
 		int iov_len,
 		int guarantee);
 
-	int (*tpg_joined_send_ok) (
-		corosync_tpg_handle handle,
+	int (*tpg_joined_reserve) (
+		hdb_handle_t handle,
 		struct iovec *iovec,
 		int iov_len);
 
+	int (*tpg_joined_release) (
+		int reserved_msgs);
+
 	int (*tpg_groups_mcast) (
-		corosync_tpg_handle handle,
+		hdb_handle_t handle,
 		int guarantee,
 		struct corosync_tpg_group *groups,
 		int groups_cnt,
 		struct iovec *iovec,
 		int iov_len);
 
-	int (*tpg_groups_send_ok) (
-		corosync_tpg_handle handle,
+	int (*tpg_groups_reserve) (
+		hdb_handle_t handle,
 		struct corosync_tpg_group *groups,
 		int groups_cnt,
 		struct iovec *iovec,
 		int iov_len);
 
+	int (*tpg_groups_release) (
+		int reserved_msgs);
+
 	int (*sync_request) (
 		char *service_name);
 
+	/*
+	 * User plugin-callable functions for quorum
+	 */
+	int (*quorum_is_quorate) (void);
+	int (*quorum_register_callback) (quorum_callback_fn_t callback_fn, void *context);
+	int (*quorum_unregister_callback) (quorum_callback_fn_t callback_fn, void *context);
+
+	/*
+	 * This one is for the quorum management plugin's use
+	 */
+	int (*quorum_initialize)(struct quorum_callin_functions *fns,
+				 sync_callback_fn_t *sync_callback_fn);
+
 	/*
 	 * Plugin loading and unloading
 	 */
 	int (*plugin_interface_reference) (
-		unsigned int *handle, 
+		hdb_handle_t *handle, 
 		char *iface_name,
 		int version,
 		void **interface,
 		void *context);
 
-	int (*plugin_interface_release) (unsigned int handle);
+	int (*plugin_interface_release) (hdb_handle_t handle);
 
 	/*
 	 * Service loading and unloading APIs
@@ -485,7 +542,7 @@ struct corosync_api_v1 {
 	 */
 	void (*error_memory_failure) (void);
 #define corosync_fatal_error(err) api->fatal_error ((err), __FILE__, __LINE__)
-	void (*fatal_error) (corosync_fatal_error_t err, const char *file, unsigned int line);
+	void (*fatal_error) (cs_fatal_error_t err, const char *file, unsigned int line);
 };
 
 #define SERVICE_ID_MAKE(a,b) ( ((a)<<16) | (b) )
@@ -496,7 +553,7 @@ struct corosync_lib_handler {
 	void (*lib_handler_fn) (void *conn, void *msg);
 	int response_size;
 	int response_id;
-	enum corosync_lib_flow_control flow_control;
+	enum cs_lib_flow_control flow_control;
 };
 
 struct corosync_exec_handler {
@@ -512,7 +569,8 @@ struct corosync_service_engine {
 	char *name;
 	unsigned short id;
 	unsigned int private_data_size;
-	enum corosync_lib_flow_control flow_control;
+	enum cs_lib_flow_control flow_control;
+	enum cs_lib_allow_inquorate allow_inquorate;
 	int (*exec_init_fn) (struct corosync_api_v1 *);
 	int (*exec_exit_fn) (void);
 	void (*exec_dump_fn) (void);
