@@ -35,7 +35,7 @@
 #define IPC_CONFDB_H_DEFINED
 
 #include <netinet/in.h>
-#include "saAis.h"
+#include <corosync/corotypes.h>
 #include "ipc_gen.h"
 
 enum req_confdb_types {
@@ -84,48 +84,48 @@ enum res_confdb_types {
 
 struct req_lib_confdb_object_create {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
 	mar_name_t object_name __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_object_create {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_object_destroy {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_object_parent_get {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_object_parent_get {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
 };
 
 
 struct req_lib_confdb_key_create {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 	mar_name_t key_name __attribute__((aligned(8)));
 	mar_name_t value __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_key_delete {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 	mar_name_t key_name __attribute__((aligned(8)));
 	mar_name_t value __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_key_replace {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 	mar_name_t key_name __attribute__((aligned(8)));
 	mar_name_t old_value __attribute__((aligned(8)));
 	mar_name_t new_value __attribute__((aligned(8)));
@@ -133,34 +133,34 @@ struct req_lib_confdb_key_replace {
 
 struct req_lib_confdb_object_find {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
 	mar_name_t object_name __attribute__((aligned(8)));
-	mar_uint32_t find_handle __attribute__((aligned(8)));
+	mar_uint64_t find_handle __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_object_find {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
-	mar_uint32_t find_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t find_handle __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_object_iter {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
-	mar_uint32_t find_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t find_handle __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_object_iter {
 	mar_res_header_t header __attribute__((aligned(8)));
 	mar_name_t object_name __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
-	mar_uint32_t find_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t find_handle __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_key_iter {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
-	mar_uint32_t next_entry __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t next_entry __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_key_iter {
@@ -171,13 +171,13 @@ struct res_lib_confdb_key_iter {
 
 struct req_lib_confdb_key_get {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
 	mar_name_t key_name __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_object_find_destroy {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t find_handle __attribute__((aligned(8)));
+	mar_uint64_t find_handle __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_key_get {
@@ -207,9 +207,9 @@ struct res_lib_confdb_reload {
 
 struct res_lib_confdb_key_change_callback {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t change_type __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t change_type __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 	mar_name_t object_name __attribute__((aligned(8)));
 	mar_name_t key_name __attribute__((aligned(8)));
 	mar_name_t key_value __attribute__((aligned(8)));
@@ -217,20 +217,20 @@ struct res_lib_confdb_key_change_callback {
 
 struct res_lib_confdb_object_create_callback {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 	mar_name_t name __attribute__((aligned(8)));
 };
 
 struct res_lib_confdb_object_destroy_callback {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t parent_object_handle __attribute__((aligned(8)));
+	mar_uint64_t parent_object_handle __attribute__((aligned(8)));
 	mar_name_t name __attribute__((aligned(8)));
 };
 
 struct req_lib_confdb_object_track_start {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_uint32_t object_handle __attribute__((aligned(8)));
+	mar_uint64_t object_handle __attribute__((aligned(8)));
 	mar_uint32_t flags __attribute__((aligned(8)));
 };
 
