@@ -35,7 +35,7 @@
 #define IPC_CPG_H_DEFINED
 
 #include <netinet/in.h>
-#include "saAis.h"
+#include <corosync/corotypes.h>
 #include "ipc_gen.h"
 #include "mar_cpg.h"
 
@@ -122,7 +122,6 @@ struct req_lib_cpg_mcast {
 
 struct res_lib_cpg_mcast {
 	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t flow_control_state __attribute__((aligned(8)));
 };
 
 /* Message from another node */
@@ -132,7 +131,6 @@ struct res_lib_cpg_deliver_callback {
 	mar_uint32_t msglen __attribute__((aligned(8)));
 	mar_uint32_t nodeid __attribute__((aligned(8)));
 	mar_uint32_t pid __attribute__((aligned(8)));
-	mar_uint32_t flow_control_state __attribute__((aligned(8)));
 	mar_uint8_t message[] __attribute__((aligned(8)));
 };
 
@@ -143,7 +141,7 @@ struct res_lib_cpg_flowcontrol_callback {
 
 struct req_lib_cpg_membership {
 	mar_req_header_t header __attribute__((aligned(8)));
-	mar_cpg_name_t group_name __attribute__((aligned(8)));
+//	mar_cpg_name_t group_name __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_confchg_callback {
