@@ -5,7 +5,7 @@
  *
  * All rights reserved.
  *
- * Author: Christine Caulfield (ccaulfie@redhat.com)
+ * Author: Patrick Caulfield (pcaulfie@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -34,9 +34,11 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Provides a closed process group API using the corosync executive
+ * Provides a closed process group API using the coroipcc executive
  */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -45,17 +47,15 @@
 #include <sys/socket.h>
 #include <errno.h>
 
-#include <corosync/saAis.h>
+#include <corosync/corotypes.h>
 #include <corosync/cpg.h>
 #include <corosync/ipc_cpg.h>
 #include <corosync/mar_cpg.h>
-#include <corosync/ais_util.h>
+#include <corosync/coroipcc.h>
 
 struct cpg_inst {
-	int response_fd;
-	int dispatch_fd;
+	void *ipc_ctx;
 	int finalize;
-	cpg_flow_control_state_t flow_control_state;
 	cpg_callbacks_t callbacks;
 	void *context;
 	pthread_mutex_t response_mutex;
@@ -72,7 +72,7 @@ static struct saHandleDatabase cpg_handle_t_db = {
 };
 
 /*
- * Clean up function for a cpg instance (cpg_initialize) handle
+ * Clean up function for a cpg instance (cpg_nitialize) handle
  */
 static void cpg_instance_destructor (void *instance)
 {
@@ -84,33 +84,31 @@ static void cpg_instance_destructor (void *instance)
 
 
 /**
- * @defgroup cpg_corosync The closed process group API
- * @ingroup corosync
+ * @defgroup cpg_coroipcc The closed process group API
+ * @ingroup coroipcc
  *
  * @{
  */
 
-cpg_error_t cpg_initialize (
+cs_error_t cpg_initialize (
 	cpg_handle_t *handle,
 	cpg_callbacks_t *callbacks)
 {
-	SaAisErrorT error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
 	error = saHandleCreate (&cpg_handle_t_db, sizeof (struct cpg_inst), handle);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, *handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_destroy;
 	}
 
-	error = saServiceConnect (&cpg_inst->dispatch_fd,
-				     &cpg_inst->response_fd,
-		CPG_SERVICE);
-	if (error != SA_AIS_OK) {
+	error = coroipcc_service_connect (IPC_SOCKET_NAME, CPG_SERVICE, &cpg_inst->ipc_ctx);
+	if (error != CS_OK) {
 		goto error_put_destroy;
 	}
 
@@ -122,7 +120,7 @@ cpg_error_t cpg_initialize (
 
 	saHandleInstancePut (&cpg_handle_t_db, *handle);
 
-	return (SA_AIS_OK);
+	return (CS_OK);
 
 error_put_destroy:
 	saHandleInstancePut (&cpg_handle_t_db, *handle);
@@ -132,14 +130,14 @@ error_no_destroy:
 	return (error);
 }
 
-cpg_error_t cpg_finalize (
+cs_error_t cpg_finalize (
 	cpg_handle_t handle)
 {
 	struct cpg_inst *cpg_inst;
-	SaAisErrorT error;
+	cs_error_t error;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -156,54 +154,45 @@ cpg_error_t cpg_finalize (
 
 	cpg_inst->finalize = 1;
 
+	coroipcc_service_disconnect (cpg_inst->ipc_ctx);
+
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
 
 	saHandleDestroy (&cpg_handle_t_db, handle);
 
-	/*
-	 * Disconnect from the server
-	 */
-	if (cpg_inst->response_fd != -1) {
-		shutdown(cpg_inst->response_fd, 0);
-		close(cpg_inst->response_fd);
-	}
-	if (cpg_inst->dispatch_fd != -1) {
-		shutdown(cpg_inst->dispatch_fd, 0);
-		close(cpg_inst->dispatch_fd);
-	}
 	saHandleInstancePut (&cpg_handle_t_db, handle);
 
 	return (CPG_OK);
 }
 
-cpg_error_t cpg_fd_get (
+cs_error_t cpg_fd_get (
 	cpg_handle_t handle,
 	int *fd)
 {
-	SaAisErrorT error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
-	*fd = cpg_inst->dispatch_fd;
+	*fd = coroipcc_fd_get (cpg_inst->ipc_ctx);
 
 	saHandleInstancePut (&cpg_handle_t_db, handle);
 
-	return (SA_AIS_OK);
+	return (CS_OK);
 }
 
-cpg_error_t cpg_context_get (
+cs_error_t cpg_context_get (
 	cpg_handle_t handle,
 	void **context)
 {
-	SaAisErrorT error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -211,18 +200,18 @@ cpg_error_t cpg_context_get (
 
 	saHandleInstancePut (&cpg_handle_t_db, handle);
 
-	return (SA_AIS_OK);
+	return (CS_OK);
 }
 
-cpg_error_t cpg_context_set (
+cs_error_t cpg_context_set (
 	cpg_handle_t handle,
 	void *context)
 {
-	SaAisErrorT error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -230,7 +219,7 @@ cpg_error_t cpg_context_set (
 
 	saHandleInstancePut (&cpg_handle_t_db, handle);
 
-	return (SA_AIS_OK);
+	return (CS_OK);
 }
 
 struct res_overlay {
@@ -238,20 +227,17 @@ struct res_overlay {
 	char data[512000];
 };
 
-cpg_error_t cpg_dispatch (
+cs_error_t cpg_dispatch (
 	cpg_handle_t handle,
-	cpg_dispatch_t dispatch_types)
+	cs_dispatch_flags_t dispatch_types)
 {
-	struct pollfd ufds;
 	int timeout = -1;
-	SaAisErrorT error;
+	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
 	int dispatch_avail;
 	struct cpg_inst *cpg_inst;
-	struct res_lib_cpg_flowcontrol_callback *res_cpg_flowcontrol_callback;
 	struct res_lib_cpg_confchg_callback *res_cpg_confchg_callback;
 	struct res_lib_cpg_deliver_callback *res_cpg_deliver_callback;
-	struct res_lib_cpg_groups_get_callback *res_lib_cpg_groups_get_callback;
 	cpg_callbacks_t callbacks;
 	struct res_overlay dispatch_data;
 	int ignore_dispatch = 0;
@@ -264,7 +250,7 @@ cpg_error_t cpg_dispatch (
 	unsigned int i;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -277,35 +263,17 @@ cpg_error_t cpg_dispatch (
 	}
 
 	do {
-		ufds.fd = cpg_inst->dispatch_fd;
-		ufds.events = POLLIN;
-		ufds.revents = 0;
-
-		error = saPollRetry (&ufds, 1, timeout);
-		if (error != SA_AIS_OK) {
-			goto error_nounlock;
-		}
-
 		pthread_mutex_lock (&cpg_inst->dispatch_mutex);
 
-		/*
-		 * Regather poll data in case ufds has changed since taking lock
-		 */
-		error = saPollRetry (&ufds, 1, timeout);
-		if (error != SA_AIS_OK) {
-			goto error_nounlock;
+		dispatch_avail = coroipcc_dispatch_recv (cpg_inst->ipc_ctx,
+			(void *)&dispatch_data, timeout);
+
+		pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
+
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (cpg_inst->finalize == 1) {
-			error = CPG_OK;
-			pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
-			goto error_unlock;
-		}
-
-		dispatch_avail = ufds.revents & POLLIN;
 		if (dispatch_avail == 0 && dispatch_types == CPG_DISPATCH_ALL) {
 			pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
 			break; /* exit do while cont is 1 loop */
@@ -314,37 +282,21 @@ cpg_error_t cpg_dispatch (
 			pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
 			continue; /* next poll */
 		}
-
-		if (ufds.revents & POLLIN) {
-			/*
-			 * Queue empty, read response from socket
-			 */
-			error = saRecvRetry (cpg_inst->dispatch_fd, &dispatch_data.header,
-				sizeof (mar_res_header_t));
-			if (error != SA_AIS_OK) {
-				goto error_unlock;
+		if (dispatch_avail == -1) {
+			if (cpg_inst->finalize == 1) {
+				error = CS_OK;
+			} else {
+				error = CS_ERR_LIBRARY;
 			}
-			if (dispatch_data.header.size > sizeof (mar_res_header_t)) {
-				error = saRecvRetry (cpg_inst->dispatch_fd, &dispatch_data.data,
-					dispatch_data.header.size - sizeof (mar_res_header_t));
-
-				if (error != SA_AIS_OK) {
-					goto error_unlock;
-				}
-			}
-		} else {
-			pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
-			continue;
+			goto error_put;
 		}
 
 		/*
 		 * Make copy of callbacks, message data, unlock instance, and call callback
 		 * A risk of this dispatch method is that the callback routines may
 		 * operate at the same time that cpgFinalize has been called.
-		*/
+		 */
 		memcpy (&callbacks, &cpg_inst->callbacks, sizeof (cpg_callbacks_t));
-
-		pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
 		/*
 		 * Dispatch incoming message
 		 */
@@ -352,7 +304,6 @@ cpg_error_t cpg_dispatch (
 		case MESSAGE_RES_CPG_DELIVER_CALLBACK:
 			res_cpg_deliver_callback = (struct res_lib_cpg_deliver_callback *)&dispatch_data;
 
-			cpg_inst->flow_control_state = res_cpg_deliver_callback->flow_control_state;
 			marshall_from_mar_cpg_name_t (
 				&group_name,
 				&res_cpg_deliver_callback->group_name);
@@ -399,33 +350,9 @@ cpg_error_t cpg_dispatch (
 				res_cpg_confchg_callback->joined_list_entries);
 			break;
 
-		case MESSAGE_RES_CPG_GROUPS_CALLBACK:
-			res_lib_cpg_groups_get_callback = (struct res_lib_cpg_groups_get_callback *)&dispatch_data;
-			marshall_from_mar_cpg_name_t (
-				&group_name,
-				&res_lib_cpg_groups_get_callback->group_name);
-			for (i = 0; i < res_lib_cpg_groups_get_callback->num_members; i++) {
-				marshall_from_mar_cpg_address_t (&member_list[i],
-					&res_lib_cpg_groups_get_callback->member_list[i]);
-			}
-
-			callbacks.cpg_groups_get_fn(handle,
-						    res_lib_cpg_groups_get_callback->group_num,
-						    res_lib_cpg_groups_get_callback->total_groups,
-						    &group_name,
-						    member_list,
-						    res_lib_cpg_groups_get_callback->num_members);
-
-			break;
-
-		case MESSAGE_RES_CPG_FLOWCONTROL_CALLBACK:
-			res_cpg_flowcontrol_callback = (struct res_lib_cpg_flowcontrol_callback *)&dispatch_data;
-			cpg_inst->flow_control_state = res_cpg_flowcontrol_callback->flow_control_state;
-			break;
-
 		default:
-			error = SA_AIS_ERR_LIBRARY;
-			goto error_nounlock;
+			error = CS_ERR_LIBRARY;
+			goto error_put;
 			break;
 		}
 
@@ -450,17 +377,16 @@ cpg_error_t cpg_dispatch (
 		}
 	} while (cont);
 
-error_unlock:
+error_put:
 	saHandleInstancePut (&cpg_handle_t_db, handle);
-error_nounlock:
 	return (error);
 }
 
-cpg_error_t cpg_join (
+cs_error_t cpg_join (
     cpg_handle_t handle,
     struct cpg_name *group)
 {
-	cpg_error_t error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 	struct iovec iov[2];
 	struct req_lib_cpg_join req_lib_cpg_join;
@@ -469,7 +395,7 @@ cpg_error_t cpg_join (
 	struct res_lib_cpg_trackstart res_lib_cpg_trackstart;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -481,13 +407,13 @@ cpg_error_t cpg_join (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_trackstart.group_name,
 		group);
 
-	iov[0].iov_base = (char *)&req_lib_cpg_trackstart;
+	iov[0].iov_base = &req_lib_cpg_trackstart;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_trackstart);
 
-	error = saSendMsgReceiveReply (cpg_inst->dispatch_fd, iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov, 1,
 		&res_lib_cpg_trackstart, sizeof (struct res_lib_cpg_trackstart));
 
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		pthread_mutex_unlock (&cpg_inst->response_mutex);
 		goto error_exit;
 	}
@@ -499,15 +425,15 @@ cpg_error_t cpg_join (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_join.group_name,
 		group);
 
-	iov[0].iov_base = (char *)&req_lib_cpg_join;
+	iov[0].iov_base = &req_lib_cpg_join;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_join);
 
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov, 1,
 		&res_lib_cpg_join, sizeof (struct res_lib_cpg_join));
 
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
 
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_exit;
 	}
 
@@ -519,18 +445,18 @@ error_exit:
 	return (error);
 }
 
-cpg_error_t cpg_leave (
+cs_error_t cpg_leave (
     cpg_handle_t handle,
     struct cpg_name *group)
 {
-	cpg_error_t error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 	struct iovec iov[2];
 	struct req_lib_cpg_leave req_lib_cpg_leave;
 	struct res_lib_cpg_leave res_lib_cpg_leave;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -540,16 +466,16 @@ cpg_error_t cpg_leave (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_leave.group_name,
 		group);
 
-	iov[0].iov_base = (char *)&req_lib_cpg_leave;
+	iov[0].iov_base = &req_lib_cpg_leave;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_leave);
 
 	pthread_mutex_lock (&cpg_inst->response_mutex);
 
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov, 1,
 		&res_lib_cpg_leave, sizeof (struct res_lib_cpg_leave));
 
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_exit;
 	}
 
@@ -561,14 +487,14 @@ error_exit:
 	return (error);
 }
 
-cpg_error_t cpg_mcast_joined (
+cs_error_t cpg_mcast_joined (
 	cpg_handle_t handle,
 	cpg_guarantee_t guarantee,
 	struct iovec *iovec,
 	int iov_len)
 {
 	int i;
-	cpg_error_t error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 	struct iovec iov[64];
 	struct req_lib_cpg_mcast req_lib_cpg_mcast;
@@ -576,7 +502,7 @@ cpg_error_t cpg_mcast_joined (
 	int msg_len = 0;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -591,30 +517,21 @@ cpg_error_t cpg_mcast_joined (
 	req_lib_cpg_mcast.guarantee = guarantee;
 	req_lib_cpg_mcast.msglen = msg_len;
 
-	iov[0].iov_base = (char *)&req_lib_cpg_mcast;
+	iov[0].iov_base = &req_lib_cpg_mcast;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_mcast);
 	memcpy (&iov[1], iovec, iov_len * sizeof (struct iovec));
 
 	pthread_mutex_lock (&cpg_inst->response_mutex);
 
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, iov, iov_len + 1,
-		&res_lib_cpg_mcast, sizeof (res_lib_cpg_mcast));
+	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov,
+		iov_len + 1, &res_lib_cpg_mcast, sizeof (res_lib_cpg_mcast));
 
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
 
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_exit;
 	}
 
-/*	Only update the flow control state when the return value is OK.
- *	Otherwise the flow control state is not guaranteed to be valid in the
- *	return message.
- *	Also, don't set to ENABLED if the return value is TRY_AGAIN as this can lead
- *	to Flow Control State sync issues between AIS LIB and EXEC.
- */
-	if (res_lib_cpg_mcast.header.error == CPG_OK) {
-		cpg_inst->flow_control_state = res_lib_cpg_mcast.flow_control_state;
-	}
 	error = res_lib_cpg_mcast.header.error;
 
 error_exit:
@@ -623,13 +540,13 @@ error_exit:
 	return (error);
 }
 
-cpg_error_t cpg_membership_get (
+cs_error_t cpg_membership_get (
 	cpg_handle_t handle,
 	struct cpg_name *group_name,
 	struct cpg_address *member_list,
 	int *member_list_entries)
 {
-	cpg_error_t error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 	struct iovec iov;
 	struct req_lib_cpg_membership req_lib_cpg_membership_get;
@@ -637,26 +554,24 @@ cpg_error_t cpg_membership_get (
 	unsigned int i;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
 	req_lib_cpg_membership_get.header.size = sizeof (mar_req_header_t);
 	req_lib_cpg_membership_get.header.id = MESSAGE_REQ_CPG_MEMBERSHIP;
-	marshall_to_mar_cpg_name_t (&req_lib_cpg_membership_get.group_name,
-		group_name);
 
-	iov.iov_base = (char *)&req_lib_cpg_membership_get;
+	iov.iov_base = &req_lib_cpg_membership_get;
 	iov.iov_len = sizeof (mar_req_header_t);
 
 	pthread_mutex_lock (&cpg_inst->response_mutex);
 
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, &iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, &iov, 1,
 		&res_lib_cpg_membership_get, sizeof (mar_res_header_t));
 
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
 
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_exit;
 	}
 
@@ -679,18 +594,18 @@ error_exit:
 	return (error);
 }
 
-cpg_error_t cpg_local_get (
+cs_error_t cpg_local_get (
 	cpg_handle_t handle,
 	unsigned int *local_nodeid)
 {
-	cpg_error_t error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 	struct iovec iov;
 	struct req_lib_cpg_local_get req_lib_cpg_local_get;
 	struct res_lib_cpg_local_get res_lib_cpg_local_get;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
 
@@ -702,12 +617,12 @@ cpg_error_t cpg_local_get (
 
 	pthread_mutex_lock (&cpg_inst->response_mutex);
 
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, &iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, &iov, 1,
 		&res_lib_cpg_local_get, sizeof (res_lib_cpg_local_get));
 
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
 
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		goto error_exit;
 	}
 
@@ -721,61 +636,19 @@ error_exit:
 	return (error);
 }
 
-cpg_error_t cpg_groups_get (
-	cpg_handle_t handle,
-	unsigned int *num_groups)
-{
-	cpg_error_t error;
-	struct cpg_inst *cpg_inst;
-	struct iovec iov;
-	struct req_lib_cpg_groups_get req_lib_cpg_groups_get;
-	struct res_lib_cpg_groups_get res_lib_cpg_groups_get;
-
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
-		return (error);
-	}
-
-	req_lib_cpg_groups_get.header.size = sizeof (mar_req_header_t);
-	req_lib_cpg_groups_get.header.id = MESSAGE_REQ_CPG_GROUPS_GET;
-
-	iov.iov_base = &req_lib_cpg_groups_get;
-	iov.iov_len = sizeof (struct req_lib_cpg_groups_get);
-
-	pthread_mutex_lock (&cpg_inst->response_mutex);
-
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, &iov, 1,
-		&res_lib_cpg_groups_get, sizeof (res_lib_cpg_groups_get));
-
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-
-	if (error != SA_AIS_OK) {
-		goto error_exit;
-	}
-
-	*num_groups = res_lib_cpg_groups_get.num_groups;
-	error = res_lib_cpg_groups_get.header.error;
-
-	/* Real output is delivered via a callback */
-error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
-
-	return (error);
-}
-
-cpg_error_t cpg_flow_control_state_get (
+cs_error_t cpg_flow_control_state_get (
 	cpg_handle_t handle,
 	cpg_flow_control_state_t *flow_control_state)
 {
-	cpg_error_t error;
+	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
 	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
+	if (error != CS_OK) {
 		return (error);
 	}
-
-	*flow_control_state = cpg_inst->flow_control_state;
+	
+	*flow_control_state = coroipcc_dispatch_flow_control_get (cpg_inst->ipc_ctx);
 
 	saHandleInstancePut (&cpg_handle_t_db, handle);
 
