@@ -31,6 +31,9 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#include <config.h>
+
 #ifndef COROSYNC_BSD
 #include <alloca.h>
 #endif
@@ -51,7 +54,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <corosync/saAis.h>
+#include <corosync/corotypes.h>
 #include <corosync/cfg.h>
 #include <corosync/list.h>
 #include <corosync/queue.h>
@@ -99,7 +102,6 @@ struct process_info {
 	void *conn;
 	void *trackerconn;
 	struct group_info *group;
-	enum corosync_flow_control_state flow_control_state;
 	struct list_head list; /* on the group_info members list */
 };
 
@@ -189,49 +191,49 @@ static struct corosync_lib_handler cpg_lib_engine[] =
 		.lib_handler_fn				= message_handler_req_lib_cpg_join,
 		.response_size				= sizeof (struct res_lib_cpg_join),
 		.response_id				= MESSAGE_RES_CPG_JOIN,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_REQUIRED
 	},
 	{ /* 1 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_leave,
 		.response_size				= sizeof (struct res_lib_cpg_leave),
 		.response_id				= MESSAGE_RES_CPG_LEAVE,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_REQUIRED
 	},
 	{ /* 2 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_mcast,
 		.response_size				= sizeof (struct res_lib_cpg_mcast),
 		.response_id				= MESSAGE_RES_CPG_MCAST,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_REQUIRED
 	},
 	{ /* 3 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_membership,
 		.response_size				= sizeof (mar_res_header_t),
 		.response_id				= MESSAGE_RES_CPG_MEMBERSHIP,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 4 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_trackstart,
 		.response_size				= sizeof (struct res_lib_cpg_trackstart),
 		.response_id				= MESSAGE_RES_CPG_TRACKSTART,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 5 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_trackstop,
 		.response_size				= sizeof (struct res_lib_cpg_trackstart),
 		.response_id				= MESSAGE_RES_CPG_TRACKSTOP,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 6 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_local_get,
 		.response_size				= sizeof (struct res_lib_cpg_local_get),
 		.response_id				= MESSAGE_RES_CPG_LOCAL_GET,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 7 */
 		.lib_handler_fn				= message_handler_req_lib_cpg_groups_get,
 		.response_size				= sizeof (struct res_lib_cpg_groups_get),
 		.response_id				= MESSAGE_RES_CPG_GROUPS_GET,
-		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
+		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	}
 };
 
@@ -263,7 +265,8 @@ struct corosync_service_engine cpg_service_engine = {
 	.name				        = "corosync cluster closed process group service v1.01",
 	.id					= CPG_SERVICE,
 	.private_data_size			= sizeof (struct process_info),
-	.flow_control				= COROSYNC_LIB_FLOW_CONTROL_REQUIRED,
+	.flow_control				= CS_LIB_FLOW_CONTROL_REQUIRED,
+	.allow_inquorate			= CS_LIB_ALLOW_INQUORATE,
 	.lib_init_fn				= cpg_lib_init_fn,
 	.lib_exit_fn				= cpg_lib_exit_fn,
 	.lib_engine				= cpg_lib_engine,
@@ -395,7 +398,7 @@ static int notify_lib_joinlist(
 		sizeof(mar_cpg_address_t) * (count + left_list_entries + joined_list_entries);
 	buf = alloca(size);
 	if (!buf)
-		return SA_AIS_ERR_NO_SPACE;
+		return CS_ERR_NO_SPACE;
 
 	res = (struct res_lib_cpg_confchg_callback *)buf;
 	res->joined_list_entries = joined_list_entries;
@@ -404,6 +407,7 @@ static int notify_lib_joinlist(
 
 	res->header.size = size;
 	res->header.id = id;
+	res->header.error = CS_OK;
 	memcpy(&res->group_name, &gi->group_name, sizeof(mar_cpg_name_t));
 
 	/* Build up the message */
@@ -439,21 +443,21 @@ static int notify_lib_joinlist(
 	}
 
 	if (conn) {
-		api->ipc_conn_send_response(conn, buf, size);
+		api->ipc_dispatch_send(conn, buf, size);
 	}
 	else {
 		/* Send it to all listeners */
 		for (iter = gi->members.next, tmp=iter->next; iter != &gi->members; iter = tmp, tmp=iter->next) {
 			struct process_info *pi = list_entry(iter, struct process_info, list);
 			if (pi->trackerconn && (pi->flags & PI_FLAG_MEMBER)) {
-				if (api->ipc_conn_send_response(pi->trackerconn, buf, size) == -1) {
+				if (api->ipc_dispatch_send(pi->trackerconn, buf, size) == -1) {
 					// Error ??
 				}
 			}
 		}
 	}
 
-	return SA_AIS_OK;
+	return CS_OK;
 }
 
 static void remove_group(struct group_info *gi)
@@ -488,8 +492,10 @@ static int cpg_lib_exit_fn (void *conn)
 		notify_info.nodeid = api->totem_nodeid_get();
 		notify_info.reason = CONFCHG_CPG_REASON_PROCDOWN;
 		cpg_node_joinleave_send(gi, pi, MESSAGE_REQ_EXEC_CPG_PROCLEAVE, CONFCHG_CPG_REASON_PROCDOWN);
-		list_del(&pi->list);
 	}
+	if (pi->pid)
+		list_del(&pi->list);
+	api->ipc_refcnt_dec (conn);
 	return (0);
 }
 
@@ -536,70 +542,6 @@ static struct group_info *get_group(mar_cpg_name_t *name)
 	return gi;
 }
 
-static void send_group_list_callbacks(int num_groups, void *conn)
-{
-	struct list_head *iter, *piter;
-	struct group_info *gi;
-	uint32_t hash;
-	int max_proc_count=0;
-	int size = 0;
-	int group_counter = 0;
-	char *buf = NULL;
-	struct res_lib_cpg_groups_get_callback *res;
-	mar_cpg_address_t *retgi;
-
-	for (hash=0; hash < GROUP_HASH_SIZE; hash++) {
-		for (iter = group_lists[hash].next; iter != &group_lists[hash]; iter = iter->next) {
-			gi = list_entry(iter, struct group_info, list);
-			int proc_count = 0;
-
-			/* First, we need to know how many processes are in the list */
-			for (piter = gi->members.next; piter != &gi->members; piter = piter->next) {
-				struct process_info *pi = list_entry(piter, struct process_info, list);
-				if (pi->pid)
-					proc_count++;
-			}
-
-			/* Make sure we have adequate buffer space */
-			if (proc_count > max_proc_count) {
-				max_proc_count = proc_count+10;
-				size = max_proc_count*sizeof(mar_cpg_address_t) +
-					sizeof(struct res_lib_cpg_groups_get_callback);
-				buf = realloc(buf, size);
-				if (!buf) {
-					log_printf(LOG_LEVEL_WARNING, "Unable to allocate group_list struct");
-					return;
-				}
-			}
-
-			res = (struct res_lib_cpg_groups_get_callback *)buf;
-			retgi = res->member_list;
-
-			res->header.size = size;
-			res->header.id = MESSAGE_RES_CPG_GROUPS_CALLBACK;
-
-
-			memcpy(&res->group_name, &gi->group_name, sizeof(mar_cpg_name_t));
-			res->num_members = proc_count;
-			res->group_num = ++group_counter;
-			res->total_groups = num_groups;
-
-			for (piter = gi->members.next; piter != &gi->members; piter = piter->next) {
-				struct process_info *pi = list_entry(piter, struct process_info, list);
-				if (pi->pid) {
-					retgi->nodeid = pi->nodeid;
-					retgi->pid = pi->pid;
-					retgi->reason = 0;
-					retgi++;
-				}
-			}
-			api->ipc_conn_send_response(conn, buf, size);
-		}
-	}
-	if (buf)
-		free(buf);
-}
-
 static int cpg_node_joinleave_send (struct group_info *gi, struct process_info *pi, int fn, int reason)
 {
 	struct req_exec_cpg_procjoin req_exec_cpg_procjoin;
@@ -694,7 +636,7 @@ static void cpg_confchg_fn (
 	struct memb_ring_id *ring_id)
 {
 	int i;
-	uint32_t lowest_nodeid = 0xffffff;
+	uint32_t lowest_nodeid = 0xffffffff;
 	struct iovec req_exec_cpg_iovec;
 
 	/* We don't send the library joinlist in here because it can end up
@@ -734,34 +676,6 @@ static void cpg_confchg_fn (
 	}
 }
 
-static void cpg_flow_control_state_set_fn (
-	void *context,
-	enum corosync_flow_control_state flow_control_state)
-{
-	struct res_lib_cpg_flowcontrol_callback res_lib_cpg_flowcontrol_callback;
-	struct process_info *process_info = (struct process_info *)context;
-
-	process_info->flow_control_state = flow_control_state;
-	/*
-	 * Send disabled flow control if a disabled occurs.  This prevents
-	 * the condition where a disabled occurs after all messages have been
-	 * delivered and then there is no valid way to retrieve the flow
-	 * control state
-	 */
-	if (flow_control_state == CPG_FLOW_CONTROL_DISABLED) {
-		res_lib_cpg_flowcontrol_callback.header.id = MESSAGE_RES_CPG_FLOWCONTROL_CALLBACK;
-		res_lib_cpg_flowcontrol_callback.header.size = sizeof (struct res_lib_cpg_flowcontrol_callback);
-		res_lib_cpg_flowcontrol_callback.flow_control_state = flow_control_state;
-
-		if (process_info->trackerconn) {
-			api->ipc_response_no_fcc (
-				process_info->trackerconn,
-				&res_lib_cpg_flowcontrol_callback,
-				sizeof (struct res_lib_cpg_flowcontrol_callback));
-		}
-	}
-}
-
 /* Can byteswap join & leave messages */
 static void exec_cpg_procjoin_endian_convert (void *msg)
 {
@@ -1001,7 +915,6 @@ static void message_handler_req_exec_cpg_mcast (
 	res_lib_cpg_mcast->msglen = msglen;
 	res_lib_cpg_mcast->pid = req_exec_cpg_mcast->pid;
 	res_lib_cpg_mcast->nodeid = nodeid;
-	res_lib_cpg_mcast->flow_control_state = CPG_FLOW_CONTROL_DISABLED;
 	if (api->ipc_source_is_local (&req_exec_cpg_mcast->source)) {
 		api->ipc_refcnt_dec (req_exec_cpg_mcast->source.conn);
 	}
@@ -1014,8 +927,7 @@ static void message_handler_req_exec_cpg_mcast (
 	for (iter = gi->members.next; iter != &gi->members; iter = iter->next) {
 		struct process_info *pi = list_entry(iter, struct process_info, list);
 		if (pi->trackerconn && (pi->flags & PI_FLAG_MEMBER)) {
-			res_lib_cpg_mcast->flow_control_state = pi->flow_control_state;
-			api->ipc_conn_send_response(
+			api->ipc_dispatch_send(
 				pi->trackerconn,
 				buf,
 				res_lib_cpg_mcast->header.size);
@@ -1092,6 +1004,7 @@ static int cpg_exec_send_joinlist(void)
 static int cpg_lib_init_fn (void *conn)
 {
 	struct process_info *pi = (struct process_info *)api->ipc_private_data_get (conn);
+	api->ipc_refcnt_inc (conn);
 	pi->conn = conn;
 
 	log_printf(LOG_LEVEL_DEBUG, "lib_init_fn: conn=%p, pi=%p\n", conn, pi);
@@ -1105,30 +1018,22 @@ static void message_handler_req_lib_cpg_join (void *conn, void *message)
 	struct process_info *pi = (struct process_info *)api->ipc_private_data_get (conn);
 	struct res_lib_cpg_join res_lib_cpg_join;
 	struct group_info *gi;
-	SaAisErrorT error = SA_AIS_OK;
+	cs_error_t error = CS_OK;
 
 	log_printf(LOG_LEVEL_DEBUG, "got join request on %p, pi=%p, pi->pid=%d\n", conn, pi, pi->pid);
 
 	/* Already joined on this conn */
 	if (pi->pid) {
-		error = SA_AIS_ERR_INVALID_PARAM;
+		error = CS_ERR_INVALID_PARAM;
 		goto join_err;
 	}
 
 	gi = get_group(&req_lib_cpg_join->group_name);
 	if (!gi) {
-		error = SA_AIS_ERR_NO_SPACE;
+		error = CS_ERR_NO_SPACE;
 		goto join_err;
 	}
 
-	api->ipc_fc_create (
-		conn,
-		CPG_SERVICE,
-		req_lib_cpg_join->group_name.value,
-		req_lib_cpg_join->group_name.length,
-		cpg_flow_control_state_set_fn,
-		pi);
-
 	/* Add a node entry for us */
 	pi->nodeid = api->totem_nodeid_get();
 	pi->pid = req_lib_cpg_join->pid;
@@ -1142,7 +1047,7 @@ join_err:
 	res_lib_cpg_join.header.size = sizeof(res_lib_cpg_join);
 	res_lib_cpg_join.header.id = MESSAGE_RES_CPG_JOIN;
 	res_lib_cpg_join.header.error = error;
-	api->ipc_conn_send_response(conn, &res_lib_cpg_join, sizeof(res_lib_cpg_join));
+	api->ipc_response_send(conn, &res_lib_cpg_join, sizeof(res_lib_cpg_join));
 }
 
 /* Leave message from the library */
@@ -1151,12 +1056,12 @@ static void message_handler_req_lib_cpg_leave (void *conn, void *message)
 	struct process_info *pi = (struct process_info *)api->ipc_private_data_get (conn);
 	struct res_lib_cpg_leave res_lib_cpg_leave;
 	struct group_info *gi;
-	SaAisErrorT error = SA_AIS_OK;
+	cs_error_t error = CS_OK;
 
 	log_printf(LOG_LEVEL_DEBUG, "got leave request on %p\n", conn);
 
 	if (!pi || !pi->pid || !pi->group) {
-		error = SA_AIS_ERR_INVALID_PARAM;
+		error = CS_ERR_INVALID_PARAM;
 		goto leave_ret;
 	}
 	gi = pi->group;
@@ -1166,18 +1071,12 @@ static void message_handler_req_lib_cpg_leave (void *conn, void *message)
 	cpg_node_joinleave_send(gi, pi, MESSAGE_REQ_EXEC_CPG_PROCLEAVE, CONFCHG_CPG_REASON_LEAVE);
 	pi->group = NULL;
 
-	api->ipc_fc_destroy (
-		conn,
-		CPG_SERVICE,
-		(unsigned char *)gi->group_name.value,
-		(unsigned int)gi->group_name.length);
-
 leave_ret:
 	/* send return */
 	res_lib_cpg_leave.header.size = sizeof(res_lib_cpg_leave);
 	res_lib_cpg_leave.header.id = MESSAGE_RES_CPG_LEAVE;
 	res_lib_cpg_leave.header.error = error;
-	api->ipc_conn_send_response(conn, &res_lib_cpg_leave, sizeof(res_lib_cpg_leave));
+	api->ipc_response_send(conn, &res_lib_cpg_leave, sizeof(res_lib_cpg_leave));
 }
 
 /* Mcast message from the library */
@@ -1198,9 +1097,8 @@ static void message_handler_req_lib_cpg_mcast (void *conn, void *message)
 	if (!gi) {
 		res_lib_cpg_mcast.header.size = sizeof(res_lib_cpg_mcast);
 		res_lib_cpg_mcast.header.id = MESSAGE_RES_CPG_MCAST;
-		res_lib_cpg_mcast.header.error = SA_AIS_ERR_ACCESS; /* TODO Better error code ?? */
-		res_lib_cpg_mcast.flow_control_state = CPG_FLOW_CONTROL_DISABLED;
-		api->ipc_conn_send_response(conn, &res_lib_cpg_mcast,
+		res_lib_cpg_mcast.header.error = CS_ERR_ACCESS; /* TODO Better error code ?? */
+		api->ipc_response_send(conn, &res_lib_cpg_mcast,
 			sizeof(res_lib_cpg_mcast));
 		return;
 	}
@@ -1225,9 +1123,8 @@ static void message_handler_req_lib_cpg_mcast (void *conn, void *message)
 
 	res_lib_cpg_mcast.header.size = sizeof(res_lib_cpg_mcast);
 	res_lib_cpg_mcast.header.id = MESSAGE_RES_CPG_MCAST;
-	res_lib_cpg_mcast.header.error = SA_AIS_OK;
-	res_lib_cpg_mcast.flow_control_state = pi->flow_control_state;
-	api->ipc_conn_send_response(conn, &res_lib_cpg_mcast,
+	res_lib_cpg_mcast.header.error = CS_OK;
+	api->ipc_response_send(conn, &res_lib_cpg_mcast,
 		sizeof(res_lib_cpg_mcast));
 }
 
@@ -1240,8 +1137,8 @@ static void message_handler_req_lib_cpg_membership (void *conn, void *message)
 		mar_res_header_t res;
 		res.size = sizeof(res);
 		res.id = MESSAGE_RES_CPG_MEMBERSHIP;
-		res.error = SA_AIS_ERR_ACCESS; /* TODO Better error code */
-		api->ipc_conn_send_response(conn, &res, sizeof(res));
+		res.error = CS_ERR_ACCESS; /* TODO Better error code */
+		api->ipc_response_send(conn, &res, sizeof(res));
 		return;
 	}
 
@@ -1255,27 +1152,25 @@ static void message_handler_req_lib_cpg_trackstart (void *conn, void *message)
 	struct res_lib_cpg_trackstart res_lib_cpg_trackstart;
 	struct group_info *gi;
 	struct process_info *otherpi;
-	void *otherconn;
-	SaAisErrorT error = SA_AIS_OK;
+	cs_error_t error = CS_OK;
 
 	log_printf(LOG_LEVEL_DEBUG, "got trackstart request on %p\n", conn);
 
 	gi = get_group(&req_lib_cpg_trackstart->group_name);
 	if (!gi) {
-		error = SA_AIS_ERR_NO_SPACE;
+		error = CS_ERR_NO_SPACE;
 		goto tstart_ret;
 	}
 
 	/* Find the partner connection and add us to it's process_info struct */
-	otherconn = api->ipc_conn_partner_get (conn);
 	otherpi = (struct process_info *)api->ipc_private_data_get (conn);
 	otherpi->trackerconn = conn;
 
 tstart_ret:
 	res_lib_cpg_trackstart.header.size = sizeof(res_lib_cpg_trackstart);
 	res_lib_cpg_trackstart.header.id = MESSAGE_RES_CPG_TRACKSTART;
-	res_lib_cpg_trackstart.header.error = SA_AIS_OK;
-	api->ipc_conn_send_response(conn, &res_lib_cpg_trackstart, sizeof(res_lib_cpg_trackstart));
+	res_lib_cpg_trackstart.header.error = CS_OK;
+	api->ipc_response_send(conn, &res_lib_cpg_trackstart, sizeof(res_lib_cpg_trackstart));
 }
 
 static void message_handler_req_lib_cpg_trackstop (void *conn, void *message)
@@ -1283,28 +1178,26 @@ static void message_handler_req_lib_cpg_trackstop (void *conn, void *message)
 	struct req_lib_cpg_trackstop *req_lib_cpg_trackstop = (struct req_lib_cpg_trackstop *)message;
 	struct res_lib_cpg_trackstop res_lib_cpg_trackstop;
 	struct process_info *otherpi;
-	void *otherconn;
 	struct group_info *gi;
-	SaAisErrorT error = SA_AIS_OK;
+	cs_error_t error = CS_OK;
 
 	log_printf(LOG_LEVEL_DEBUG, "got trackstop request on %p\n", conn);
 
 	gi = get_group(&req_lib_cpg_trackstop->group_name);
 	if (!gi) {
-		error = SA_AIS_ERR_NO_SPACE;
+		error = CS_ERR_NO_SPACE;
 		goto tstop_ret;
 	}
 
 	/* Find the partner connection and add us to it's process_info struct */
-	otherconn = api->ipc_conn_partner_get (conn);
 	otherpi = (struct process_info *)api->ipc_private_data_get (conn);
 	otherpi->trackerconn = NULL;
 
 tstop_ret:
 	res_lib_cpg_trackstop.header.size = sizeof(res_lib_cpg_trackstop);
 	res_lib_cpg_trackstop.header.id = MESSAGE_RES_CPG_TRACKSTOP;
-	res_lib_cpg_trackstop.header.error = SA_AIS_OK;
-	api->ipc_conn_send_response(conn, &res_lib_cpg_trackstop.header, sizeof(res_lib_cpg_trackstop));
+	res_lib_cpg_trackstop.header.error = CS_OK;
+	api->ipc_response_send(conn, &res_lib_cpg_trackstop.header, sizeof(res_lib_cpg_trackstop));
 }
 
 static void message_handler_req_lib_cpg_local_get (void *conn, void *message)
@@ -1313,10 +1206,10 @@ static void message_handler_req_lib_cpg_local_get (void *conn, void *message)
 
 	res_lib_cpg_local_get.header.size = sizeof(res_lib_cpg_local_get);
 	res_lib_cpg_local_get.header.id = MESSAGE_RES_CPG_LOCAL_GET;
-	res_lib_cpg_local_get.header.error = SA_AIS_OK;
+	res_lib_cpg_local_get.header.error = CS_OK;
 	res_lib_cpg_local_get.local_nodeid = api->totem_nodeid_get ();
 
-	api->ipc_conn_send_response(conn, &res_lib_cpg_local_get,
+	api->ipc_response_send(conn, &res_lib_cpg_local_get,
 		sizeof(res_lib_cpg_local_get));
 }
 
@@ -1326,14 +1219,10 @@ static void message_handler_req_lib_cpg_groups_get (void *conn, void *message)
 
 	res_lib_cpg_groups_get.header.size = sizeof(res_lib_cpg_groups_get);
 	res_lib_cpg_groups_get.header.id = MESSAGE_RES_CPG_GROUPS_GET;
-	res_lib_cpg_groups_get.header.error = SA_AIS_OK;
+	res_lib_cpg_groups_get.header.error = CS_OK;
 	res_lib_cpg_groups_get.num_groups = count_groups();
 
-	api->ipc_conn_send_response(conn, &res_lib_cpg_groups_get,
+	api->ipc_response_send(conn, &res_lib_cpg_groups_get,
 		sizeof(res_lib_cpg_groups_get));
-
-	/* Now do the callbacks for each group */
-	send_group_list_callbacks(res_lib_cpg_groups_get.num_groups,
-		api->ipc_conn_partner_get (conn));
 }
 
